
val hang : unit -> bool
type domUserData = string
type nodetype =
    BLANK_NODE
  | ELEMENT_NODE
  | ATTRIBUTE_NODE
  | TEXT_NODE
  | CDATA_SECTION_NODE
  | ENTITY_REFERENCE_NODE
  | ENTITY_NODE
  | PROCESSING_INSTRUCTION_NODE
  | COMMENT_NODE
  | DOCUMENT_NODE
  | DOCUMENT_TYPE_NODE
  | DOCUMENT_FRAGMENT_NODE
  | NOTATION_NODE
  | XPATH_NAMESPACE_NODE
type eventPhase = START_PHASE | CAPTURE_PHASE | TARGET_PHASE | BUBBLE_PHASE | DEFAULT_PHASE
type eventType =
  | BLANK_EVENT
  | MOUSEDOWN
  | MOUSEUP
  | MOUSEOVER
  | MOUSEOUT
  | MOUSEMOVE
  | MOUSEDRAG
  | CLICK
  | DBLCLICK
  | KEYDOWN
  | KEYUP
  | KEYPRESS
  | DRAGDROP
  | FOCUS
  | BLUR
  | SELECT
  | CHANGE

class labelClass :
  object
    val mutable label : int
    method getLabel : int
    method isEqual : labelClass -> bool
    method isGr : labelClass -> bool
    method isLE : labelClass -> bool
    method isNotEqual : labelClass -> bool
    method join : labelClass -> labelClass
    method setLabel : labelClass -> unit
  end
val pcLabel : labelClass
type domString = string
type labeledDomString = string * labelClass
val nsuCheck : labelClass -> labelClass -> bool
class jsFunctionObject :
  object
    val mutable functionObjectLabel : labelClass
    method getfunctionObjectLabel : labelClass
    method setfunctionObjectLabel : labelClass -> unit
  end
class nativeFunctionObject :
  object
    method run : unit -> bool
  end
class event :
  object
    val mutable bubbles : bool
    val mutable cancelable : bool
    val mutable currentTarget : eventTarget
    (* val mutable defaultPrevented : bool *)
    val mutable dispatched : bool
    val mutable eventLabel : labelClass
    val mutable evnType : eventType
    val mutable phase : eventPhase
    (* val mutable propagationStopped : bool *)
    (* val mutable stopImmediatePropagation : bool *)
    val mutable target : eventTarget
    val mutable timestamp: float
    method defaultPrevented : bool
    method canBubble : bool
    method getPhase: eventPhase
    method setPhase: eventPhase -> unit
    method getTimestamp: float
    method setTimestamp: float -> unit
    method getCurrentTarget : eventTarget
    method getEventType : (eventType*labelClass)
    method getTarget : eventTarget
    method getEventLabel : labelClass
    method getStopImmediatePropagation : bool
    method initEvent : eventType -> bool -> bool -> unit
    method preventDefault : unit
    method setCurrentTarget : eventTarget -> unit
    method setEventType : (eventType) -> unit
    method setTarget : (eventTarget) -> unit
    method setEventLabel : labelClass -> unit
    method stopImmediatePropagation : unit -> unit
    method stopPropagation : unit
    method getStopPropagation : bool
  end
and uiEvent :
  object
    inherit event
  end
and focusEvent : 
  object
    inherit uiEvent
  end
and sharedKeyboardMouseEvent :
  object
    inherit uiEvent
    method initSharedKeyboardMouseEvent : eventType -> domString
    method getModifierState : domString -> bool
  end
and mouseEvent : 
  object
    inherit sharedKeyboardMouseEvent
  end
and wheelEvent : 
  object
    inherit mouseEvent
  end
and inputEvent : 
  object
    inherit uiEvent
  end
and keyboardEvent : 
  object
    inherit sharedKeyboardMouseEvent
  end
and compositionEvent : 
  object
    inherit uiEvent
  end
and eventListener :
  object
    val mutable eventType : eventType
    val mutable f : jsFunctionObject
    val mutable useCapture : bool
    method getEventType : eventType
    method handleEvent : event -> unit
    method private invokeJsc : unit
    method setCapture : bool -> unit
    method setEventType : eventType -> unit
  end
and eventTarget :
  object
    val mutable eventTargetLabel : labelClass
    val mutable eventListenerMap :
      (eventType, (eventListener * bool) list) Hashtbl.t
    val mutable defaulteventListenerMap :
      (eventType, nativeFunctionObject) Hashtbl.t
    (* method getEventTargetLabel: labelClass  *)
    (* method setEventTargetLabel: labelClass -> unit *)
    method addEventListener : eventType -> eventListener -> bool -> unit
    method dispatchEvent : event -> node' -> bool
    method private existInlist :
      (eventListener * bool) list -> eventListener -> bool -> bool
    method getEventHandlers : eventType -> (eventListener*bool) list
    method private getEventPathToDocumentNode : node' -> node' list
    method getEventListenerMap :
      (eventType, (eventListener * bool) list) Hashtbl.t
    method removeEventListener : eventType -> eventListener -> bool -> unit
    method private removeFromList :
      (eventListener * bool) list ->
      eventListener -> bool -> (eventListener * bool) list
    method setEventListenerMap :
      (eventType, (eventListener * bool) list) Hashtbl.t -> unit
  end
and virtual newnodeList :
  object
    method virtual item : int -> (node' option * labelClass)
    method virtual length : (int * labelClass)
  end
and childNodeList :
  node' ->
  object
    inherit newnodeList
    val ownerNode : node'
    method item : int -> (node' option * labelClass)
    method length : (int * labelClass)
  end
and tagNodeList :
  node' ->
  domString ->
  object
    inherit newnodeList
    val rootNode : node'
    val tName : domString
    method item : int -> (node' option * labelClass)
    method length : (int * labelClass)
  end
and nsNodeList :
  node' ->
  domString ->
  domString ->
  object
    inherit newnodeList
    val nsName : domString
    val lName : domString
    val rootNode : node'
    method item : int -> (node' option * labelClass)
    method length : (int * labelClass)
  end
and node' :
  object
    inherit eventTarget
    val mutable baseURI : domString
    val mutable childNodes : childNodeList
    val mutable docLabel : labelClass
    val mutable eventListenerMap :
      (eventType, (eventListener * bool) list) Hashtbl.t
    val mutable firstChild : node' option
    val mutable firstChildLabel : labelClass
    val mutable isContainerNode : bool
    val mutable lastChild : node' option
    val mutable lastChildLabel : labelClass
    val mutable localName : domString
    val mutable mattr : attrnamedNodeMap
    val mutable nType : nodetype
    val mutable namespaceURI : domString
    val mutable nextSibling : node' option
    val mutable nextSiblingLabel : labelClass
    val mutable nodeLabel : labelClass
    val mutable nodeName : domString
    val mutable nodeValue : labeledDomString
    val mutable nodetagName : domString
    val mutable ownerDocument : document
    val mutable parentNode : node' option
    val mutable parentNodeLabel : labelClass
    val mutable prefix : domString
    val mutable previousSibling : node' option
    val mutable previousSiblingLabel : labelClass
    val mutable qualifiedName : domString
    (* val mutable udHandler : userDataHandler *)
    val mutable userData : (domString, domUserData) Hashtbl.t
    method appendChild : node' -> bool
    method cloneNode : bool -> node'
    method cloneSubTree : node'
    method cloneWithChildren : node'
    method cloneWithoutChildren : node'
    method dispatchEvent : event -> node' -> bool
    method private existInlist :
      (eventListener * bool) list -> eventListener -> bool -> bool
    method getAttr : attrnamedNodeMap
    method getAttribute : domString -> (domString * labelClass) option
    method getChildList : childNodeList
    method getEventHandlers : eventType -> (eventListener*bool) list
    method getNextSibling : (node' option*labelClass)
    method getNodeTagName : (domString*labelClass)
    method getPreviousSibling : (node' option*labelClass)
    method getUserData : domString -> domUserData * labelClass
    method getbaseURI : domString * labelClass
    method getDocLabel : labelClass
    method getEventListenerMap :
      (eventType, (eventListener * bool) list) Hashtbl.t
    method getFirstChild : (node' option * labelClass)
    method getFirstChildLabel : labelClass
    method getLastChild : (node' option * labelClass)
    method getLastChildLabel : labelClass
    method getLocalName : domString * labelClass
    method getnType : (nodetype*labelClass)
    method getNamespaceURI : domString * labelClass
    method getNextSiblingLabel : labelClass
    method getNodeLabel : labelClass
    method getNodeName : domString * labelClass
    method getNodeValue : labeledDomString
    method getOwnerDocument : (document*labelClass)
    method getParentNode : (node' option * labelClass)
    method getParentNodeLabel : labelClass
    method getprefix : domString * labelClass
    method getPreviousSiblingLabel : labelClass
    method getqualifiedName : domString * labelClass
    method hasAttributes : bool
    method hasChildNodes : bool * labelClass
    method insertBefore : node' -> node' option -> bool
    method isEqualNode : node' -> bool
    method private isParent : node' -> bool
    method isSameNode : node' -> bool 
    method mergeEmptyAndAdjacentTextNodes : unit
    method normalize : unit -> unit
    method removeChild : node' -> bool
    method removeEventListener : eventType -> eventListener -> bool -> unit
    method private removeFromList :
      (eventListener * bool) list ->
      eventListener -> bool -> (eventListener * bool) list
    method private removeNodeWithChildren : node' -> unit
    method replaceChild : node' -> node' -> bool
    method setContainerNode : bool -> unit
    method setNextSibling : (node' option) -> unit
    method setNodeTagName : (domString*labelClass) -> unit
    method setPreviousSibling : (node' option) -> unit
    method setUserData : domString -> domUserData -> unit
    method setbaseURI : (domString*labelClass) -> unit
    method setDocLabel : labelClass -> unit
    method setEventListenerMap :
      (eventType, (eventListener * bool) list) Hashtbl.t -> unit
    method setFirstChild : node' option -> unit
    method setFirstChildLabel : labelClass -> unit
    method setLastChild : (node' option) -> unit
    method setLastChildLabel : labelClass -> unit
    method setlocalName : (domString*labelClass) -> unit
    method setnType : (nodetype*labelClass) -> unit
    method setNamespaceURI : domString -> unit
    method setNextSiblingLabel : labelClass -> unit
    method setNodeLabel : labelClass -> unit
    method setNodeName : (domString *labelClass) -> unit
    method setNodeValue : labeledDomString -> unit
    method setOwnerDocument : (document) -> unit
    method setParentNode : (node' option) -> unit
    method setParentNodeLabel : labelClass -> unit
    method setprefix : (domString*labelClass) -> unit
    method setPreviousSiblingLabel : labelClass -> unit
    method setqualifiedName : (domString*labelClass) -> unit
  end
(* and userDataHandler : *)
(*   object *)
(*     val mutable f : unit -> unit *)
(*     method getf : unit -> unit *)
(*     method handle : int -> domString -> domUserData -> node' -> node' -> unit *)
(*     method setUserDataHandler : userDataHandler -> unit *)
(*     method setf : (unit -> unit) -> unit *)
(*   end *)
and attr' :
  object
    inherit node'
    val mutable attrName : domString
    val mutable baseURI : domString
    val mutable childNodes : childNodeList
    val mutable docLabel : labelClass
    val mutable eventListenerMap :
      (eventType, (eventListener * bool) list) Hashtbl.t
    val mutable firstChild : node' option
    val mutable firstChildLabel : labelClass
    val mutable isContainerNode : bool
    val mutable lastChild : node' option
    val mutable lastChildLabel : labelClass
    val mutable localName : domString
    val mutable mattr : attrnamedNodeMap
    val mutable nType : nodetype
    val mutable namespaceURI : domString
    val mutable nextSibling : node' option
    val mutable nextSiblingLabel : labelClass
    val mutable nodeLabel : labelClass
    val mutable nodeName : domString
    val mutable nodeValue : labeledDomString
    val mutable nodetagName : domString
    val mutable ownerDocument : document
    val mutable parentNode : node' option
    val mutable parentNodeLabel : labelClass
    val mutable prefix : domString
    val mutable previousSibling : node' option
    val mutable previousSiblingLabel : labelClass
    val mutable qualifiedName : domString
    (* val mutable udHandler : userDataHandler *)
    val mutable userData : (domString, domUserData) Hashtbl.t
    val mutable value : domString
    method addEventListener : eventType -> eventListener -> bool -> unit
    method appendChild : node' -> bool
    method cloneNode : bool -> node'
    method cloneSubTree : node'
    method cloneWithChildren : node'
    method cloneWithoutChildren : node'
    method dispatchEvent : event -> node' -> bool
    method private existInlist :
      (eventListener * bool) list -> eventListener -> bool -> bool
    method getChildList : childNodeList
    method getEventHandlers : eventType -> (eventListener*bool) list
    method getName : labeledDomString
    method getNextSibling : (node' option*labelClass)
    method getNodeTagName : (domString*labelClass)
    method getPreviousSibling : (node' option*labelClass)
    method getUserData : domString -> domUserData * labelClass
    method getVal : labeledDomString
    method getbaseURI : domString * labelClass
    method getDocLabel : labelClass
    method getEventListenerMap :
      (eventType, (eventListener * bool) list) Hashtbl.t
    method getFirstChild : (node' option * labelClass)
    method getFirstChildLabel : labelClass
    method getLastChild : (node' option * labelClass)
    method getLastChildLabel : labelClass
    method getLocalName : domString * labelClass
    method getnType : (nodetype*labelClass)
    method getNamespaceURI : domString * labelClass
    method getNextSiblingLabel : labelClass
    method getNodeLabel : labelClass
    method getNodeName : domString * labelClass
    method getNodeValue : labeledDomString
    method getOwnerDocument : (document*labelClass)
    method getParentNode : (node' option * labelClass)
    method getParentNodeLabel : labelClass
    method getprefix : domString * labelClass
    method getPreviousSiblingLabel : labelClass
    method getqualifiedName : domString * labelClass
    method hasAttributes : bool
    method hasChildNodes : bool * labelClass
    method insertBefore : node' -> node' option -> bool
    method isEqualNode : node' -> bool
    method private isParent : node' -> bool
    method isSameNode : node' -> bool
    method mergeEmptyAndAdjacentTextNodes : unit
    method normalize : unit -> unit
    method removeChild : node' -> bool
    method removeEventListener : eventType -> eventListener -> bool -> unit
    method private removeFromList :
      (eventListener * bool) list ->
      eventListener -> bool -> (eventListener * bool) list
    method private removeNodeWithChildren : node' -> unit
    method replaceChild : node' -> node' -> bool
    method setContainerNode : bool -> unit
    method setName : (domString*labelClass) -> unit
    method setNextSibling : (node' option) -> unit
    method setNodeTagName : (domString*labelClass) -> unit
    method setPreviousSibling : (node' option) -> unit
    method setUserData : domString -> domUserData -> unit
    method setVal : (domString*labelClass) -> unit
    method setbaseURI : (domString*labelClass) -> unit
    method setDocLabel : labelClass -> unit
    method setEventListenerMap :
      (eventType, (eventListener * bool) list) Hashtbl.t -> unit
    method setFirstChild : node' option -> unit
    method setFirstChildLabel : labelClass -> unit
    method setLastChild : (node' option) -> unit
    method setLastChildLabel : labelClass -> unit
    method setlocalName : (domString*labelClass) -> unit
    method setnType : (nodetype*labelClass) -> unit
    method setNamespaceURI : domString -> unit
    method setNextSiblingLabel : labelClass -> unit
    method setNodeLabel : labelClass -> unit
    method setNodeName : (domString*labelClass) -> unit
    method setNodeValue : labeledDomString -> unit
    method setOwnerDocument : (document) -> unit
    method setParentNode : (node' option) -> unit
    method setParentNodeLabel : labelClass -> unit
    method setprefix : (domString*labelClass) -> unit
    method setPreviousSiblingLabel : labelClass -> unit
    method setqualifiedName : (domString*labelClass) -> unit
  end
and element' :
  domString ->
  object
    inherit node'
    val mutable baseURI : domString
    val mutable childNodes : childNodeList
    val mutable docLabel : labelClass
    val mutable eventListenerMap :
      (eventType, (eventListener * bool) list) Hashtbl.t
    val mutable firstChild : node' option
    val mutable firstChildLabel : labelClass
    val mutable isContainerNode : bool
    val mutable lastChild : node' option
    val mutable lastChildLabel : labelClass
    val mutable localName : domString
    val mutable mattr : attrnamedNodeMap
    val mutable nType : nodetype
    val mutable namespaceURI : domString
    val mutable nextSibling : node' option
    val mutable nextSiblingLabel : labelClass
    val mutable nodeLabel : labelClass
    val mutable nodeName : domString
    val mutable nodeValue : labeledDomString
    val mutable nodetagName : domString
    val mutable ownerDocument : document
    val mutable parentNode : node' option
    val mutable parentNodeLabel : labelClass
    val mutable prefix : domString
    val mutable previousSibling : node' option
    val mutable previousSiblingLabel : labelClass
    val mutable qualifiedName : domString
    (* val mutable udHandler : userDataHandler *)
    val mutable userData : (domString, domUserData) Hashtbl.t
    method addEventListener : eventType -> eventListener -> bool -> unit
    method appendChild : node' -> bool
    method cloneNode : bool -> node'
    method cloneSubTree : node'
    method cloneWithChildren : node'
    method cloneWithoutChildren : node'
    method dispatchEvent : event -> node' -> bool
    method private existInlist :
      (eventListener * bool) list -> eventListener -> bool -> bool
    method getAttribute : domString -> (domString * labelClass) option
    method getAttributeNS : domString -> domString -> (domString * labelClass) option
    method getAttributeNode : domString -> node' option
    method getAttributeNodeNS : domString -> domString -> node' option
    method getChildList : childNodeList
    method getElementsByTagName : domString -> tagNodeList
    method getElementsByTagNameNS : domString -> domString-> nsNodeList
    method getEventHandlers : eventType -> (eventListener*bool) list
    method getNextSibling : (node' option*labelClass)
    method getNodeTagName : (domString*labelClass)
    method getPreviousSibling : (node' option*labelClass)
    method getUserData : domString -> domUserData * labelClass
    method getbaseURI : domString * labelClass
    method getDocLabel : labelClass
    method getEventListenerMap :
      (eventType, (eventListener * bool) list) Hashtbl.t
    method getFirstChild : (node' option * labelClass)
    method getFirstChildLabel : labelClass
    method getLastChild : (node' option * labelClass)
    method getLastChildLabel : labelClass
    method getLocalName : domString * labelClass
    method getnType : (nodetype*labelClass)
    method getNamespaceURI : domString * labelClass
    method getNextSiblingLabel : labelClass
    method getNodeLabel : labelClass
    method getNodeName : domString * labelClass
    method getNodeValue : labeledDomString
    method getOwnerDocument : (document*labelClass)
    method getParentNode : (node' option * labelClass)
    method getParentNodeLabel : labelClass
    method getprefix : domString * labelClass
    method getPreviousSiblingLabel : labelClass
    method getqualifiedName : domString * labelClass
    method hasAttribute : domString -> bool
    method hasChildNodes : bool * labelClass
    method insertBefore : node' -> node' option -> bool
    method isEqualNode : node' -> bool
    method private isParent : node' -> bool
    method isSameNode : node' -> bool
    method mergeEmptyAndAdjacentTextNodes : unit
    method normalize : unit -> unit
    method removeAttribute : domString -> unit
    method removeAttributeNS : domString -> domString -> unit
    method removeAttributeNode : attr' -> node' option
    method removeChild : node' -> bool
    method removeEventListener : eventType -> eventListener -> bool -> unit
    method private removeFromList :
      (eventListener * bool) list ->
      eventListener -> bool -> (eventListener * bool) list
    method private removeNodeWithChildren : node' -> unit
    method replaceChild : node' -> node' -> bool
    method setAttribute : domString -> domString -> unit
    method setAttributeNS : domString -> domString -> domString -> unit
    method setAttributeNode : attr' -> node' option
    method setAttributeNodeNS : attr' -> node' option
    method setContainerNode : bool -> unit
    method setNextSibling : (node' option) -> unit
    method setNodeTagName : (domString*labelClass) -> unit
    method setPreviousSibling : (node' option) -> unit
    method setUserData : domString -> domUserData -> unit
    method setbaseURI : (domString*labelClass) -> unit
    method setDocLabel : labelClass -> unit
    method setEventListenerMap :
      (eventType, (eventListener * bool) list) Hashtbl.t -> unit
    method setFirstChild : node' option -> unit
    method setFirstChildLabel : labelClass -> unit
    method setLastChild : (node' option) -> unit
    method setLastChildLabel : labelClass -> unit
    method setlocalName : (domString*labelClass) -> unit
    method setnType : (nodetype*labelClass) -> unit
    method setNamespaceURI : domString -> unit
    method setNextSiblingLabel : labelClass -> unit
    method setNodeLabel : labelClass -> unit
    method setNodeName : (domString * labelClass) -> unit
    method setNodeValue : labeledDomString -> unit
    method setOwnerDocument : (document) -> unit
    method setParentNode : (node' option) -> unit
    method setParentNodeLabel : labelClass -> unit
    method setprefix : (domString*labelClass) -> unit
    method setPreviousSiblingLabel : labelClass -> unit
    method setqualifiedName : (domString*labelClass) -> unit
    method toNodeofSelf : node'
  end
and document :
  object
    inherit node'
    val mutable baseURI : domString
    val mutable childNodes : childNodeList
    val mutable docLabel : labelClass
    val mutable docType : documentType'
    val mutable docmentURI : domString
    val mutable documentElement : element'
    val mutable eventListenerMap :
      (eventType, (eventListener * bool) list) Hashtbl.t
    val mutable firstChild : node' option
    val mutable firstChildLabel : labelClass
    val mutable idElementMap : (domString, node') Hashtbl.t
    val mutable idElementMapStructLabel : labelClass
    val mutable implementation : domImplementation
    val mutable isContainerNode : bool
    val mutable lastChild : node' option
    val mutable lastChildLabel : labelClass
    val mutable localName : domString
    val mutable mattr : attrnamedNodeMap
    val mutable nType : nodetype
    val mutable namespaceURI : domString
    val mutable nextSibling : node' option
    val mutable nextSiblingLabel : labelClass
    val mutable nodeLabel : labelClass
    val mutable nodeName : domString
    val mutable nodeValue : labeledDomString
    val mutable nodetagName : domString
    val mutable ownerDocument : document
    val mutable parentNode : node' option
    val mutable parentNodeLabel : labelClass
    val mutable prefix : domString
    val mutable previousSibling : node' option
    val mutable previousSiblingLabel : labelClass
    val mutable qualifiedName : domString
    (* val mutable udHandler : userDataHandler *)
    val mutable userData : (domString, domUserData) Hashtbl.t
    method addEventListener : eventType -> eventListener -> bool -> unit
    method adoptNode : node' -> node' option
    method appendChild : node' -> bool
    method cloneNode : bool -> node'
    method cloneSubTree : node'
    method cloneWithChildren : node'
    method cloneWithoutChildren : node'
    method createAttribute : labeledDomString -> attr'
    method createCDATASection : labeledDomString -> cdataSection'
    method createComment : labeledDomString -> comment'
    method createDocumentFragment : unit -> documentFragment'
    method createElement : labeledDomString -> element'
    method createEntityReference : labeledDomString -> entityReference'
    method createProcessingInstruction : domString -> labeledDomString -> processingInstruction'
    method createTextNode : labeledDomString -> text'
    method dispatchEvent : event -> node' -> bool
    method private existInlist :
      (eventListener * bool) list -> eventListener -> bool -> bool
    method getAttribute : domString -> (domString * labelClass) option
    method getChildList : childNodeList
    method getElementById : domString -> node'
    method getElementByIdLabel : domString -> labelClass
    (* method private getElementConstructor : domString -> domString -> element' *)
    method getElementsByTagName : labeledDomString -> tagNodeList
    method getEventHandlers : eventType -> (eventListener*bool) list
    method getNextSibling : (node' option*labelClass)
    method getNodeTagName : (domString*labelClass)
    method getPreviousSibling : (node' option*labelClass)
    method getUserData : domString -> domUserData * labelClass
    method getbaseURI : domString * labelClass
    method getDocLabel : labelClass
    method getdocType : (documentType'*labelClass)
    method getEventListenerMap :
      (eventType, (eventListener * bool) list) Hashtbl.t
    method getFirstChild : (node' option * labelClass)
    method getFirstChildLabel : labelClass
    method getLastChild : (node' option * labelClass)
    method getLastChildLabel : labelClass
    method getLocalName : domString * labelClass
    method getnType : (nodetype*labelClass)
    method getNamespaceURI : domString * labelClass
    method getNextSiblingLabel : labelClass
    method getNodeLabel : labelClass
    method getNodeName : domString * labelClass
    method getNodeValue : labeledDomString
    method getOwnerDocument : (document*labelClass)
    method getParentNode : (node' option * labelClass)
    method getParentNodeLabel : labelClass
    method getprefix : domString * labelClass
    method getPreviousSiblingLabel : labelClass
    method getqualifiedName : domString * labelClass
    method hasAttributes : bool
    method hasChildNodes : bool * labelClass
    method importNode : node' -> bool -> node' option
    method insertBefore : node' -> node' option -> bool
    method insertIntoidElementMap : string -> node' -> unit
    method removeFromidElementMap : string -> unit
    method isEqualNode : node' -> bool
    method private isParent : node' -> bool
    method isSameNode : node' -> bool
    method mergeEmptyAndAdjacentTextNodes : unit
    method normalize : unit -> unit
    method normalizeDocument : unit -> unit
    method removeChild : node' -> bool
    method removeEventListener : eventType -> eventListener -> bool -> unit
    method private removeFromList :
      (eventListener * bool) list ->
      eventListener -> bool -> (eventListener * bool) list
    method private removeNode : node' -> unit
    method private removeNodeWithChildren : node' -> unit
    method replaceChild : node' -> node' -> bool
    method setContainerNode : bool -> unit
    method setNextSibling : (node' option) -> unit
    method setNodeTagName : (domString*labelClass) -> unit
    method setPreviousSibling : (node' option) -> unit
    method setUserData : domString -> domUserData -> unit
    method setbaseURI : (domString*labelClass) -> unit
    method setDocLabel : labelClass -> unit
    method setdocType : (documentType'*labelClass) -> unit
    method setEventListenerMap :
      (eventType, (eventListener * bool) list) Hashtbl.t -> unit
    method setFirstChild : node' option -> unit
    method setFirstChildLabel : labelClass -> unit
    method setLastChild : (node' option ) -> unit
    method setLastChildLabel : labelClass -> unit
    method setlocalName : (domString*labelClass) -> unit
    method setnType : (nodetype*labelClass) -> unit
    method setNamespaceURI : domString -> unit
    method setNextSiblingLabel : labelClass -> unit
    method setNodeLabel : labelClass -> unit
    method setNodeName : (domString*labelClass) -> unit
    method setNodeValue : labeledDomString -> unit
    method setOwnerDocument : (document) -> unit
    method setParentNode : (node' option) -> unit
    method setParentNodeLabel : labelClass -> unit
    method setprefix : (domString*labelClass) -> unit
    method setPreviousSiblingLabel : labelClass -> unit
    method setqualifiedName : (domString*labelClass) -> unit
    method toNode : node'
  end
and domImplementation :
  object
    method createDocument :
      domString -> domString -> documentType' -> document
    method hasFeature : domString -> domString -> bool
  end
and documentType' :
  object
    inherit node'
    val mutable baseURI : domString
    val mutable childNodes : childNodeList
    val mutable docLabel : labelClass
    val mutable entities : entitynamedNodeMap
    val mutable eventListenerMap :
      (eventType, (eventListener * bool) list) Hashtbl.t
    val mutable firstChild : node' option
    val mutable firstChildLabel : labelClass
    val mutable internalSubset : string
    val mutable isContainerNode : bool
    val mutable lastChild : node' option
    val mutable lastChildLabel : labelClass
    val mutable localName : domString
    val mutable mattr : attrnamedNodeMap
    val mutable nType : nodetype
    val mutable name : string
    val mutable namespaceURI : domString
    val mutable nextSibling : node' option
    val mutable nextSiblingLabel : labelClass
    val mutable nodeLabel : labelClass
    val mutable nodeName : domString
    val mutable nodeValue : labeledDomString
    val mutable nodetagName : domString
    val mutable notations : notationnamedNodeMap
    val mutable ownerDocument : document
    val mutable parentNode : node' option
    val mutable parentNodeLabel : labelClass
    val mutable prefix : domString
    val mutable previousSibling : node' option
    val mutable previousSiblingLabel : labelClass
    val mutable publicId : string
    val mutable qualifiedName : domString
    val mutable systemId : string
    (* val mutable udHandler : userDataHandler *)
    val mutable userData : (domString, domUserData) Hashtbl.t
    method addEventListener : eventType -> eventListener -> bool -> unit
    method appendChild : node' -> bool
    method cloneNode : bool -> node'
    method cloneSubTree : node'
    method cloneWithChildren : node'
    method cloneWithoutChildren : node'
    method dispatchEvent : event -> node' -> bool
    method private existInlist :
      (eventListener * bool) list -> eventListener -> bool -> bool
    method getChildList : childNodeList
    method getentities: (entitynamedNodeMap*labelClass)
    method setentities: (node'*labelClass) -> unit
    method getnotations: (notationnamedNodeMap*labelClass)
    method setnotations: (node'*labelClass) -> unit
    method getEventHandlers : eventType -> (eventListener*bool) list
    method getNextSibling : (node' option*labelClass)
    method getNodeTagName : (domString*labelClass)
    method getPreviousSibling : (node' option*labelClass)
    method getUserData : domString -> domUserData * labelClass
    method getbaseURI : domString * labelClass
    method getDocLabel : labelClass
    method getEventListenerMap :
      (eventType, (eventListener * bool) list) Hashtbl.t
    method getFirstChild : (node' option * labelClass)
    method getFirstChildLabel : labelClass
    method getinternalSubset : domString * labelClass
    method getLastChild : (node' option * labelClass)
    method getLastChildLabel : labelClass
    method getLocalName : domString * labelClass
    method getnType : (nodetype*labelClass)
    method getname : (domString*labelClass)
    method getNamespaceURI : domString * labelClass
    method getNextSiblingLabel : labelClass
    method getNodeLabel : labelClass
    method getNodeName : domString * labelClass
    method getNodeValue : labeledDomString
    method getOwnerDocument : (document*labelClass)
    method getParentNode : (node' option * labelClass)
    method getParentNodeLabel : labelClass
    method getprefix : domString * labelClass
    method getPreviousSiblingLabel : labelClass
    method getpublicId : (domString*labelClass)
    method getqualifiedName : domString * labelClass
    method getsystemId : domString * labelClass
    method hasAttributes : bool
    method hasChildNodes : bool * labelClass
    method insertBefore : node' -> node' option -> bool
    method isEqualNode : node' -> bool
    method private isParent : node' -> bool
    method isSameNode : node' -> bool
    method mergeEmptyAndAdjacentTextNodes : unit
    method normalize : unit -> unit
    method removeChild : node' -> bool
    method removeEventListener : eventType -> eventListener -> bool -> unit
    method private removeFromList :
      (eventListener * bool) list ->
      eventListener -> bool -> (eventListener * bool) list
    method private removeNodeWithChildren : node' -> unit
    method replaceChild : node' -> node' -> bool
    method setContainerNode : bool -> unit
    method setNextSibling : (node' option) -> unit
    method setNodeTagName : (domString*labelClass) -> unit
    method setPreviousSibling : (node' option) -> unit
    method setUserData : domString -> domUserData -> unit
    method setbaseURI : (domString*labelClass) -> unit
    method setDocLabel : labelClass -> unit
    method setEventListenerMap :
      (eventType, (eventListener * bool) list) Hashtbl.t -> unit
    method setFirstChild : node' option -> unit
    method setFirstChildLabel : labelClass -> unit
    method setinternalSubset : (domString*labelClass) -> unit
    method setLastChild : (node' option ) -> unit
    method setLastChildLabel : labelClass -> unit
    method setlocalName : (domString*labelClass) -> unit
    method setnType : (nodetype*labelClass) -> unit
    method setname : (domString*labelClass) -> unit
    method setNamespaceURI : domString -> unit
    method setNextSiblingLabel : labelClass -> unit
    method setNodeLabel : labelClass -> unit
    method setNodeName : (domString*labelClass) -> unit
    method setNodeValue : labeledDomString -> unit
    method setOwnerDocument : (document) -> unit
    method setParentNode : (node' option) -> unit
    method setParentNodeLabel : labelClass -> unit
    method setprefix : (domString*labelClass) -> unit
    method setPreviousSiblingLabel : labelClass -> unit
    method setpublicId : (domString*labelClass) -> unit
    method setqualifiedName : (domString*labelClass) -> unit
    method setsystemId : (domString*labelClass) -> unit
    method private toNode :unit -> node'
  end
and documentFragment' :
  object
    inherit node'
    val mutable baseURI : domString
    val mutable childNodes : childNodeList
    val mutable docLabel : labelClass
    val mutable eventListenerMap :
      (eventType, (eventListener * bool) list) Hashtbl.t
    val mutable firstChild : node' option
    val mutable firstChildLabel : labelClass
    val mutable isContainerNode : bool
    val mutable lastChild : node' option
    val mutable lastChildLabel : labelClass
    val mutable localName : domString
    val mutable mattr : attrnamedNodeMap
    val mutable nType : nodetype
    val mutable namespaceURI : domString
    val mutable nextSibling : node' option
    val mutable nextSiblingLabel : labelClass
    val mutable nodeLabel : labelClass
    val mutable nodeName : domString
    val mutable nodeValue : labeledDomString
    val mutable nodetagName : domString
    val mutable ownerDocument : document
    val mutable parentNode : node' option
    val mutable parentNodeLabel : labelClass
    val mutable prefix : domString
    val mutable previousSibling : node' option
    val mutable previousSiblingLabel : labelClass
    val mutable qualifiedName : domString
    (* val mutable udHandler : userDataHandler *)
    val mutable userData : (domString, domUserData) Hashtbl.t
    method addEventListener : eventType -> eventListener -> bool -> unit
    method appendChild : node' -> bool
    method cloneNode : bool -> node'
    method cloneSubTree : node'
    method cloneWithChildren : node'
    method cloneWithoutChildren : node'
    method dispatchEvent : event -> node' -> bool
    method private existInlist :
      (eventListener * bool) list -> eventListener -> bool -> bool
    method getChildList : childNodeList
    method getEventHandlers : eventType -> (eventListener*bool) list
    method getNextSibling : (node' option*labelClass)
    method getNodeTagName : (domString*labelClass)
    method getPreviousSibling : (node' option*labelClass)
    method getUserData : domString -> domUserData * labelClass
    method getbaseURI : domString * labelClass
    method getDocLabel : labelClass
    method getEventListenerMap :
      (eventType, (eventListener * bool) list) Hashtbl.t
    method getFirstChild : (node' option * labelClass)
    method getFirstChildLabel : labelClass
    method getLastChild : (node' option * labelClass)
    method getLastChildLabel : labelClass
    method getLocalName : domString * labelClass
    method getnType : (nodetype*labelClass)
    method getNamespaceURI : domString * labelClass
    method getNextSiblingLabel : labelClass
    method getNodeLabel : labelClass
    method getNodeName : domString * labelClass
    method getNodeValue : labeledDomString
    method getOwnerDocument : (document*labelClass)
    method getParentNode : (node' option * labelClass)
    method getParentNodeLabel : labelClass
    method getprefix : domString * labelClass
    method getPreviousSiblingLabel : labelClass
    method getqualifiedName : domString * labelClass
    method hasAttributes : bool
    method hasChildNodes : bool * labelClass
    method insertBefore : node' -> node' option -> bool
    method isEqualNode : node' -> bool
    method private isParent : node' -> bool
    method isSameNode : node' -> bool 
    method mergeEmptyAndAdjacentTextNodes : unit
    method normalize : unit -> unit
    method removeChild : node' -> bool
    method removeEventListener : eventType -> eventListener -> bool -> unit
    method private removeFromList :
      (eventListener * bool) list ->
      eventListener -> bool -> (eventListener * bool) list
    method private removeNodeWithChildren : node' -> unit
    method replaceChild : node' -> node' -> bool
    method setContainerNode : bool -> unit
    method setNextSibling : (node' option) -> unit
    method setNodeTagName : (domString*labelClass) -> unit
    method setPreviousSibling : (node' option) -> unit
    method setUserData : domString -> domUserData -> unit
    method setbaseURI : (domString*labelClass) -> unit
    method setDocLabel : labelClass -> unit
    method setEventListenerMap :
      (eventType, (eventListener * bool) list) Hashtbl.t -> unit
    method setFirstChild : node' option -> unit
    method setFirstChildLabel : labelClass -> unit
    method setLastChild : (node' option ) -> unit
    method setLastChildLabel : labelClass -> unit
    method setlocalName : (domString*labelClass) -> unit
    method setnType : (nodetype*labelClass) -> unit
    method setNamespaceURI : domString -> unit
    method setNextSiblingLabel : labelClass -> unit
    method setNodeLabel : labelClass -> unit
    method setNodeName : (domString*labelClass) -> unit
    method setNodeValue : labeledDomString -> unit
    method setOwnerDocument : (document) -> unit
    method setParentNode : (node' option) -> unit
    method setParentNodeLabel : labelClass -> unit
    method setprefix : (domString*labelClass) -> unit
    method setPreviousSiblingLabel : labelClass -> unit
    method setqualifiedName : (domString*labelClass) -> unit
  end
and characterData' :
  object
    inherit node'
    val mutable baseURI : domString
    val mutable childNodes : childNodeList
    val mutable data : domString
    val mutable docLabel : labelClass
    val mutable eventListenerMap :
      (eventType, (eventListener * bool) list) Hashtbl.t
    val mutable firstChild : node' option
    val mutable firstChildLabel : labelClass
    val mutable isContainerNode : bool
    val mutable lastChild : node' option
    val mutable lastChildLabel : labelClass
    val mutable localName : domString
    val mutable mattr : attrnamedNodeMap
    val mutable nType : nodetype
    val mutable namespaceURI : domString
    val mutable nextSibling : node' option
    val mutable nextSiblingLabel : labelClass
    val mutable nodeLabel : labelClass
    val mutable nodeName : domString
    val mutable nodeValue : labeledDomString
    val mutable nodetagName : domString
    val mutable ownerDocument : document
    val mutable parentNode : node' option
    val mutable parentNodeLabel : labelClass
    val mutable prefix : domString
    val mutable previousSibling : node' option
    val mutable previousSiblingLabel : labelClass
    val mutable qualifiedName : domString
    val mutable stringStructLabel : labelClass
    (* val mutable udHandler : userDataHandler *)
    val mutable userData : (domString, domUserData) Hashtbl.t
    method addEventListener : eventType -> eventListener -> bool -> unit
    method appendChild : node' -> bool
    method appendData : domString -> unit
    method cloneNode : bool -> node'
    method cloneSubTree : node'
    method cloneWithChildren : node'
    method cloneWithoutChildren : node'
    method deleteData : int -> int -> unit
    method dispatchEvent : event -> node' -> bool
    method private existInlist :
      (eventListener * bool) list -> eventListener -> bool -> bool
    method getChildList : childNodeList
    method getEventHandlers : eventType -> (eventListener*bool) list
    method getNextSibling : (node' option*labelClass)
    method getNodeTagName : (domString*labelClass)
    method getPreviousSibling : (node' option*labelClass)
    method getUserData : domString -> domUserData * labelClass
    method getbaseURI : domString * labelClass
    method getDocLabel : labelClass
    method getEventListenerMap :
      (eventType, (eventListener * bool) list) Hashtbl.t
    method getFirstChild : (node' option * labelClass)
    method getFirstChildLabel : labelClass
    method getLastChild : (node' option * labelClass)
    method getLastChildLabel : labelClass
    method getLocalName : domString * labelClass
    method getnType : (nodetype*labelClass)
    method getNamespaceURI : domString * labelClass
    method getNextSiblingLabel : labelClass
    method getNodeLabel : labelClass
    method getNodeName : domString * labelClass
    method getNodeValue : labeledDomString
    method getOwnerDocument : (document*labelClass)
    method getParentNode : (node' option * labelClass)
    method getParentNodeLabel : labelClass
    method getprefix : domString * labelClass
    method getPreviousSiblingLabel : labelClass
    method getqualifiedName : domString * labelClass
    method hasAttributes : bool
    method hasChildNodes : bool * labelClass
    method insertBefore : node' -> node' option -> bool
    method insertData : int -> domString -> unit
    method isEqualNode : node' -> bool
    method private isParent : node' -> bool
    method isSameNode : node' -> bool 
    method mergeEmptyAndAdjacentTextNodes : unit
    method normalize : unit -> unit
    method removeChild : node' -> bool
    method removeEventListener : eventType -> eventListener -> bool -> unit
    method private removeFromList :
      (eventListener * bool) list ->
      eventListener -> bool -> (eventListener * bool) list
    method private removeNodeWithChildren : node' -> unit
    method replaceChild : node' -> node' -> bool
    method replaceData : int -> int -> domString -> unit
    method setContainerNode : bool -> unit
    method setNextSibling : (node' option) -> unit
    method setNodeTagName : (domString*labelClass) -> unit
    method setPreviousSibling : (node' option) -> unit
    method setUserData : domString -> domUserData -> unit
    method setbaseURI : (domString*labelClass) -> unit
    method setDocLabel : labelClass -> unit
    method setEventListenerMap :
      (eventType, (eventListener * bool) list) Hashtbl.t -> unit
    method setFirstChild : node' option -> unit
    method setFirstChildLabel : labelClass -> unit
    method setLastChild : (node' option ) -> unit
    method setLastChildLabel : labelClass -> unit
    method setlocalName : (domString*labelClass) -> unit
    method setnType : (nodetype*labelClass) -> unit
    method setNamespaceURI : domString -> unit
    method setNextSiblingLabel : labelClass -> unit
    method setNodeLabel : labelClass -> unit
    method setNodeName : (domString*labelClass) -> unit
    method setNodeValue : labeledDomString -> unit
    method setOwnerDocument : (document) -> unit
    method setParentNode : (node' option) -> unit
    method setParentNodeLabel : labelClass -> unit
    method setprefix : (domString*labelClass) -> unit
    method setPreviousSiblingLabel : labelClass -> unit
    method setqualifiedName : (domString*labelClass) -> unit
    method substringData : int -> int -> string * labelClass
  end
and text' :
  object
    inherit characterData'
    val mutable baseURI : domString
    val mutable childNodes : childNodeList
    val mutable data : domString
    val mutable docLabel : labelClass
    val mutable eventListenerMap :
      (eventType, (eventListener * bool) list) Hashtbl.t
    val mutable firstChild : node' option
    val mutable firstChildLabel : labelClass
    val mutable isContainerNode : bool
    val mutable lastChild : node' option
    val mutable lastChildLabel : labelClass
    val mutable localName : domString
    val mutable mattr : attrnamedNodeMap
    val mutable nType : nodetype
    val mutable namespaceURI : domString
    val mutable nextSibling : node' option
    val mutable nextSiblingLabel : labelClass
    val mutable nodeLabel : labelClass
    val mutable nodeName : domString
    val mutable nodeValue : labeledDomString
    val mutable nodetagName : domString
    val mutable ownerDocument : document
    val mutable parentNode : node' option
    val mutable parentNodeLabel : labelClass
    val mutable prefix : domString
    val mutable previousSibling : node' option
    val mutable previousSiblingLabel : labelClass
    val mutable qualifiedName : domString
    val mutable stringStructLabel : labelClass
    (* val mutable udHandler : userDataHandler *)
    val mutable userData : (domString, domUserData) Hashtbl.t
    method addEventListener : eventType -> eventListener -> bool -> unit
    method appendChild : node' -> bool
    method appendData : domString -> unit
    method cloneNode : bool -> node'
    method cloneSubTree : node'
    method cloneWithChildren : node'
    method cloneWithoutChildren : node'
    method deleteData : int -> int -> unit
    method dispatchEvent : event -> node' -> bool
    method private existInlist :
      (eventListener * bool) list -> eventListener -> bool -> bool
    method getChildList : childNodeList
    method getEventHandlers : eventType -> (eventListener*bool) list
    method getNextSibling : (node' option*labelClass)
    method getNodeTagName : (domString*labelClass)
    method getPreviousSibling : (node' option*labelClass)
    method getUserData : domString -> domUserData * labelClass
    method getbaseURI : domString * labelClass
    method getDocLabel : labelClass
    method getEventListenerMap :
      (eventType, (eventListener * bool) list) Hashtbl.t
    method getFirstChild : (node' option * labelClass)
    method getFirstChildLabel : labelClass
    method getLastChild : (node' option * labelClass)
    method getLastChildLabel : labelClass
    method getLocalName : domString * labelClass
    method getnType : (nodetype*labelClass)
    method getNamespaceURI : domString * labelClass
    method getNextSiblingLabel : labelClass
    method getNodeLabel : labelClass
    method getNodeName : domString * labelClass
    method getNodeValue : labeledDomString
    method getOwnerDocument : (document*labelClass)
    method getParentNode : (node' option * labelClass)
    method getParentNodeLabel : labelClass
    method getprefix : domString * labelClass
    method getPreviousSiblingLabel : labelClass
    method getqualifiedName : domString * labelClass
    method hasAttributes : bool
    method hasChildNodes : bool * labelClass
    method insertBefore : node' -> node' option -> bool
    method insertData : int -> domString -> unit
    method isEqualNode : node' -> bool
    method private isParent : node' -> bool
    method isSameNode : node' -> bool 
    method mergeEmptyAndAdjacentTextNodes : unit
    method normalize : unit -> unit
    method removeChild : node' -> bool
    method removeEventListener : eventType -> eventListener -> bool -> unit
    method private removeFromList :
      (eventListener * bool) list ->
      eventListener -> bool -> (eventListener * bool) list
    method private removeNodeWithChildren : node' -> unit
    method replaceChild : node' -> node' -> bool
    method replaceData : int -> int -> domString -> unit
    method setContainerNode : bool -> unit
    method setNextSibling : (node' option) -> unit
    method setNodeTagName : (domString*labelClass) -> unit
    method setPreviousSibling : (node' option) -> unit
    method setUserData : domString -> domUserData -> unit
    method setbaseURI : (domString*labelClass) -> unit
    method setDocLabel : labelClass -> unit
    method setEventListenerMap :
      (eventType, (eventListener * bool) list) Hashtbl.t -> unit
    method setFirstChild : node' option -> unit
    method setFirstChildLabel : labelClass -> unit
    method setLastChild : (node' option ) -> unit
    method setLastChildLabel : labelClass -> unit
    method setlocalName : (domString*labelClass) -> unit
    method setnType : (nodetype*labelClass) -> unit
    method setNamespaceURI : domString -> unit
    method setNextSiblingLabel : labelClass -> unit
    method setNodeLabel : labelClass -> unit
    method setNodeName : (domString*labelClass) -> unit
    method setNodeValue : labeledDomString -> unit
    method setOwnerDocument : (document) -> unit
    method setParentNode : (node' option) -> unit
    method setParentNodeLabel : labelClass -> unit
    method setprefix : (domString*labelClass) -> unit
    method setPreviousSiblingLabel : labelClass -> unit
    method setqualifiedName : (domString*labelClass) -> unit
    method splitText : int -> bool
    method substringData : int -> int -> string * labelClass
    method toNode : text' -> node'
  end
and comment' :
  object
    inherit characterData'
    val mutable baseURI : domString
    val mutable childNodes : childNodeList
    val mutable data : domString
    val mutable docLabel : labelClass
    val mutable eventListenerMap :
      (eventType, (eventListener * bool) list) Hashtbl.t
    val mutable firstChild : node' option
    val mutable firstChildLabel : labelClass
    val mutable isContainerNode : bool
    val mutable lastChild : node' option
    val mutable lastChildLabel : labelClass
    val mutable localName : domString
    val mutable mattr : attrnamedNodeMap
    val mutable nType : nodetype
    val mutable namespaceURI : domString
    val mutable nextSibling : node' option
    val mutable nextSiblingLabel : labelClass
    val mutable nodeLabel : labelClass
    val mutable nodeName : domString
    val mutable nodeValue : labeledDomString
    val mutable nodetagName : domString
    val mutable ownerDocument : document
    val mutable parentNode : node' option
    val mutable parentNodeLabel : labelClass
    val mutable prefix : domString
    val mutable previousSibling : node' option
    val mutable previousSiblingLabel : labelClass
    val mutable qualifiedName : domString
    val mutable stringStructLabel : labelClass
    (* val mutable udHandler : userDataHandler *)
    val mutable userData : (domString, domUserData) Hashtbl.t
    method addEventListener : eventType -> eventListener -> bool -> unit
    method appendChild : node' -> bool
    method appendData : domString -> unit
    method cloneNode : bool -> node'
    method cloneSubTree : node'
    method cloneWithChildren : node'
    method cloneWithoutChildren : node'
    method deleteData : int -> int -> unit
    method dispatchEvent : event -> node' -> bool
    method private existInlist :
      (eventListener * bool) list -> eventListener -> bool -> bool
    method getChildList : childNodeList
    method getEventHandlers : eventType -> (eventListener*bool) list
    method getNextSibling : (node' option*labelClass)
    method getNodeTagName : (domString*labelClass)
    method getPreviousSibling : (node' option*labelClass)
    method getUserData : domString -> domUserData * labelClass
    method getbaseURI : domString * labelClass
    method getDocLabel : labelClass
    method getEventListenerMap :
      (eventType, (eventListener * bool) list) Hashtbl.t
    method getFirstChild : (node' option * labelClass)
    method getFirstChildLabel : labelClass
    method getLastChild : (node' option * labelClass)
    method getLastChildLabel : labelClass
    method getLocalName : domString * labelClass
    method getnType : (nodetype*labelClass)
    method getNamespaceURI : domString * labelClass
    method getNextSiblingLabel : labelClass
    method getNodeLabel : labelClass
    method getNodeName : domString * labelClass
    method getNodeValue : labeledDomString
    method getOwnerDocument : (document*labelClass)
    method getParentNode : (node' option * labelClass)
    method getParentNodeLabel : labelClass
    method getprefix : domString * labelClass
    method getPreviousSiblingLabel : labelClass
    method getqualifiedName : domString * labelClass
    method hasAttributes : bool
    method hasChildNodes : bool * labelClass
    method insertBefore : node' -> node' option -> bool
    method insertData : int -> domString -> unit
    method isEqualNode : node' -> bool
    method private isParent : node' -> bool
    method isSameNode : node' -> bool 
    method mergeEmptyAndAdjacentTextNodes : unit
    method normalize : unit -> unit
    method removeChild : node' -> bool
    method removeEventListener : eventType -> eventListener -> bool -> unit
    method private removeFromList :
      (eventListener * bool) list ->
      eventListener -> bool -> (eventListener * bool) list
    method private removeNodeWithChildren : node' -> unit
    method replaceChild : node' -> node' -> bool
    method replaceData : int -> int -> domString -> unit
    method setContainerNode : bool -> unit
    method setNextSibling : (node' option) -> unit
    method setNodeTagName : (domString*labelClass) -> unit
    method setPreviousSibling : (node' option) -> unit
    method setUserData : domString -> domUserData -> unit
    method setbaseURI : (domString*labelClass) -> unit
    method setDocLabel : labelClass -> unit
    method setEventListenerMap :
      (eventType, (eventListener * bool) list) Hashtbl.t -> unit
    method setFirstChild : node' option -> unit
    method setFirstChildLabel : labelClass -> unit
    method setLastChild : (node' option ) -> unit
    method setLastChildLabel : labelClass -> unit
    method setlocalName : (domString*labelClass) -> unit
    method setnType : (nodetype*labelClass) -> unit
    method setNamespaceURI : domString -> unit
    method setNextSiblingLabel : labelClass -> unit
    method setNodeLabel : labelClass -> unit
    method setNodeName : (domString*labelClass) -> unit
    method setNodeValue : labeledDomString -> unit
    method setOwnerDocument : (document) -> unit
    method setParentNode : (node' option) -> unit
    method setParentNodeLabel : labelClass -> unit
    method setprefix : (domString*labelClass) -> unit
    method setPreviousSiblingLabel : labelClass -> unit
    method setqualifiedName : (domString*labelClass) -> unit
    method substringData : int -> int -> string * labelClass
  end
and cdataSection' :
  object
    val mutable baseURI : domString
    val mutable childNodes : childNodeList
    val mutable data : domString
    val mutable docLabel : labelClass
    val mutable eventListenerMap :
      (eventType, (eventListener * bool) list) Hashtbl.t
    val mutable firstChild : node' option
    val mutable firstChildLabel : labelClass
    val mutable isContainerNode : bool
    val mutable lastChild : node' option
    val mutable lastChildLabel : labelClass
    val mutable localName : domString
    val mutable mattr : attrnamedNodeMap
    val mutable nType : nodetype
    val mutable namespaceURI : domString
    val mutable nextSibling : node' option
    val mutable nextSiblingLabel : labelClass
    val mutable nodeLabel : labelClass
    val mutable nodeName : domString
    val mutable nodeValue : labeledDomString
    val mutable nodetagName : domString
    val mutable ownerDocument : document
    val mutable parentNode : node' option
    val mutable parentNodeLabel : labelClass
    val mutable prefix : domString
    val mutable previousSibling : node' option
    val mutable previousSiblingLabel : labelClass
    val mutable qualifiedName : domString
    val mutable stringStructLabel : labelClass
    (* val mutable udHandler : userDataHandler *)
    val mutable userData : (domString, domUserData) Hashtbl.t
    method addEventListener : eventType -> eventListener -> bool -> unit
    method appendChild : node' -> bool
    method appendData : domString -> unit
    method cloneNode : bool -> node'
    method cloneSubTree : node'
    method cloneWithChildren : node'
    method cloneWithoutChildren : node'
    method deleteData : int -> int -> unit
    method dispatchEvent : event -> node' -> bool
    method private existInlist :
      (eventListener * bool) list -> eventListener -> bool -> bool
    method getAttr : attrnamedNodeMap
    method getAttribute : domString -> (domString * labelClass) option
    method getChildList : childNodeList
    method getEventHandlers : eventType -> (eventListener*bool) list
    method getNextSibling : (node' option*labelClass)
    method getNodeTagName : (domString*labelClass)
    method getPreviousSibling : (node' option*labelClass)
    method getUserData : domString -> domUserData * labelClass
    method getbaseURI : domString * labelClass
    method getDocLabel : labelClass
    method getEventListenerMap :
      (eventType, (eventListener * bool) list) Hashtbl.t
    method getFirstChild : (node' option * labelClass)
    method getFirstChildLabel : labelClass
    method getLastChild : (node' option * labelClass)
    method getLastChildLabel : labelClass
    method getLocalName : domString * labelClass
    method getnType : (nodetype*labelClass)
    method getNamespaceURI : domString * labelClass
    method getNextSiblingLabel : labelClass
    method getNodeLabel : labelClass
    method getNodeName : domString * labelClass
    method getNodeValue : labeledDomString
    method getOwnerDocument : (document*labelClass)
    method getParentNode : (node' option * labelClass)
    method getParentNodeLabel : labelClass
    method getprefix : domString * labelClass
    method getPreviousSiblingLabel : labelClass
    method getqualifiedName : domString * labelClass
    method hasAttributes : bool
    method hasChildNodes : bool * labelClass
    method insertBefore : node' -> node' option -> bool
    method insertData : int -> domString -> unit
    method isEqualNode : node' -> bool
    method private isParent : node' -> bool
    method isSameNode : node' -> bool 
    method mergeEmptyAndAdjacentTextNodes : unit
    method normalize : unit -> unit
    method removeChild : node' -> bool
    method removeEventListener : eventType -> eventListener -> bool -> unit
    method private removeFromList :
      (eventListener * bool) list ->
      eventListener -> bool -> (eventListener * bool) list
    method private removeNodeWithChildren : node' -> unit
    method replaceChild : node' -> node' -> bool
    method replaceData : int -> int -> domString -> unit
    method setContainerNode : bool -> unit
    method setNextSibling : (node' option) -> unit
    method setNodeTagName : (domString*labelClass) -> unit
    method setPreviousSibling : (node' option) -> unit
    method setUserData : domString -> domUserData -> unit
    method setbaseURI : (domString*labelClass) -> unit
    method setDocLabel : labelClass -> unit
    method setEventListenerMap :
      (eventType, (eventListener * bool) list) Hashtbl.t -> unit
    method setFirstChild : node' option -> unit
    method setFirstChildLabel : labelClass -> unit
    method setLastChild : (node' option ) -> unit
    method setLastChildLabel : labelClass -> unit
    method setlocalName : (domString*labelClass) -> unit
    method setnType : (nodetype*labelClass) -> unit
    method setNamespaceURI : domString -> unit
    method setNextSiblingLabel : labelClass -> unit
    method setNodeLabel : labelClass -> unit
    method setNodeName : (domString*labelClass) -> unit
    method setNodeValue : labeledDomString -> unit
    method setOwnerDocument : (document) -> unit
    method setParentNode : (node' option) -> unit
    method setParentNodeLabel : labelClass -> unit
    method setprefix : (domString*labelClass) -> unit
    method setPreviousSiblingLabel : labelClass -> unit
    method setqualifiedName : (domString*labelClass) -> unit
    method substringData : int -> int -> string * labelClass
  end
and newnamedNodeMap : object  end
and attrnamedNodeMap :
  node' ->
  object
    val rootNode : node'
    method possessAttributes : bool
    method getNamedItem : domString -> node' option
    method getNamedItemNS : domString -> domString -> node' option
    method item : int -> node' option
    method removeNamedItem : domString -> node' option
    method removeNamedItemNS : domString -> domString -> node' option
    method setNamedItem : node' -> node' option
    method setNamedItemNS : node' -> node' option
  end
and entitynamedNodeMap :
  node' ->
  object
    val rootNode : node'
    method getNamedItem : domString -> node' option
    method getNamedItemNS : domString -> domString -> node' option
    method item : int -> node' option
    method removeNamedItem : domString -> node' option
    method removeNamedItemNS : domString -> domString -> node' option
    method setNamedItem : node' -> node' option
    method setNamedItemNS : node' -> node' option
  end
and notationnamedNodeMap :
  node' ->
  object
    val rootNode : node'
    method getNamedItem : domString -> node' option
    method getNamedItemNS : domString -> domString -> node' option
    method item : int -> node' option
    method removeNamedItem : domString -> node' option
    method removeNamedItemNS : domString -> domString -> node' option
    method setNamedItem : node' -> node' option
    method setNamedItemNS : node' -> node' option
  end
and domStringList :
  object
    val mutable domStringList : domString list
    val mutable length : int
    method contains : domString -> bool
    method private containsInternal : domString -> domString list -> bool
    method item : int -> domString
  end
and entity :
  object
    val mutable baseURI : domString
    val mutable childNodes : childNodeList
    val mutable docLabel : labelClass
    val mutable eventListenerMap :
      (eventType, (eventListener * bool) list) Hashtbl.t
    val mutable firstChild : node' option
    val mutable firstChildLabel : labelClass
    val mutable inputEncoding : string
    val mutable isContainerNode : bool
    val mutable lastChild : node' option
    val mutable lastChildLabel : labelClass
    val mutable localName : domString
    val mutable mattr : attrnamedNodeMap
    val mutable nType : nodetype
    val mutable namespaceURI : domString
    val mutable nextSibling : node' option
    val mutable nextSiblingLabel : labelClass
    val mutable nodeLabel : labelClass
    val mutable nodeName : domString
    val mutable nodeValue : labeledDomString
    val mutable nodetagName : domString
    val mutable notationName : string
    val mutable ownerDocument : document
    val mutable parentNode : node' option
    val mutable parentNodeLabel : labelClass
    val mutable prefix : domString
    val mutable previousSibling : node' option
    val mutable previousSiblingLabel : labelClass
    val mutable publicID : string
    val mutable qualifiedName : domString
    val mutable systemID : string
    (* val mutable udHandler : userDataHandler *)
    val mutable userData : (domString, domUserData) Hashtbl.t
    val mutable xmlEncoding : string
    val mutable xmlVersion : string
    method addEventListener : eventType -> eventListener -> bool -> unit
    method appendChild : node' -> bool
    method cloneNode : bool -> node'
    method cloneSubTree : node'
    method cloneWithChildren : node'
    method cloneWithoutChildren : node'
    method dispatchEvent : event -> node' -> bool
    method private existInlist :
      (eventListener * bool) list -> eventListener -> bool -> bool
    method getChildList : childNodeList
    method getEventHandlers : eventType -> (eventListener*bool) list
    method getNextSibling : (node' option*labelClass)
    method getNodeTagName : (domString*labelClass)
    method getPreviousSibling : (node' option*labelClass)
    method getUserData : domString -> domUserData * labelClass
    method getbaseURI : domString * labelClass
    method getDocLabel : labelClass
    method getEventListenerMap :
      (eventType, (eventListener * bool) list) Hashtbl.t
    method getFirstChild : (node' option * labelClass)
    method getFirstChildLabel : labelClass
    method getinputEncoding : string * labelClass
    method getLastChild : (node' option * labelClass)
    method getLastChildLabel : labelClass
    method getLocalName : domString * labelClass
    method getnType : (nodetype*labelClass)
    method getNamespaceURI : domString * labelClass
    method getNextSiblingLabel : labelClass
    method getNodeLabel : labelClass
    method getNodeName : domString * labelClass
    method getNodeValue : labeledDomString
    method getnotationName : string * labelClass
    method getOwnerDocument : (document*labelClass)
    method getParentNode : (node' option * labelClass)
    method getParentNodeLabel : labelClass
    method getprefix : domString * labelClass
    method getPreviousSiblingLabel : labelClass
    method getpublicId : domString * labelClass
    method getqualifiedName : domString * labelClass
    method getsystemId : domString * labelClass
    method getxmlEncoding : string * labelClass
    method getxmlVersion : string * labelClass
    method hasAttributes : bool
    method hasChildNodes : bool * labelClass
    method insertBefore : node' -> node' option -> bool
    method isEqualNode : node' -> bool
    method private isParent : node' -> bool
    method isSameNode : node' -> bool 
    method mergeEmptyAndAdjacentTextNodes : unit
    method normalize : unit -> unit
    method removeChild : node' -> bool
    method removeEventListener : eventType -> eventListener -> bool -> unit
    method private removeFromList :
      (eventListener * bool) list ->
      eventListener -> bool -> (eventListener * bool) list
    method private removeNodeWithChildren : node' -> unit
    method replaceChild : node' -> node' -> bool
    method setContainerNode : bool -> unit
    method setNextSibling : (node' option) -> unit
    method setNodeTagName : (domString*labelClass) -> unit
    method setPreviousSibling : (node' option) -> unit
    method setUserData : domString -> domUserData -> unit
    method setbaseURI : (domString*labelClass) -> unit
    method setDocLabel : labelClass -> unit
    method setEventListenerMap :
      (eventType, (eventListener * bool) list) Hashtbl.t -> unit
    method setFirstChild : node' option -> unit
    method setFirstChildLabel : labelClass -> unit
    method setinputEncoding : string -> unit
    method setLastChild : (node' option ) -> unit
    method setLastChildLabel : labelClass -> unit
    method setlocalName : (domString*labelClass) -> unit
    method setnType : (nodetype*labelClass) -> unit
    method setNamespaceURI : domString -> unit
    method setNextSiblingLabel : labelClass -> unit
    method setNodeLabel : labelClass -> unit
    method setNodeName : (domString*labelClass) -> unit
    method setNodeValue : labeledDomString -> unit
    method setnotationName : string -> unit
    method setOwnerDocument : (document) -> unit
    method setParentNode : (node' option) -> unit
    method setParentNodeLabel : labelClass -> unit
    method setprefix : (domString*labelClass) -> unit
    method setPreviousSiblingLabel : labelClass -> unit
    method setpublicId : (domString*labelClass) -> unit
    method setqualifiedName : (domString*labelClass) -> unit
    method setsystemId : (domString*labelClass) -> unit
    method setxmlEncoding : string -> unit
    method setxmlVersion : string -> unit
  end
and entityReference' :
  object
    inherit node'
    val mutable baseURI : domString
    val mutable childNodes : childNodeList
    val mutable docLabel : labelClass
    val mutable eventListenerMap :
      (eventType, (eventListener * bool) list) Hashtbl.t
    val mutable firstChild : node' option
    val mutable firstChildLabel : labelClass
    val mutable isContainerNode : bool
    val mutable lastChild : node' option
    val mutable lastChildLabel : labelClass
    val mutable localName : domString
    val mutable mattr : attrnamedNodeMap
    val mutable nType : nodetype
    val mutable namespaceURI : domString
    val mutable nextSibling : node' option
    val mutable nextSiblingLabel : labelClass
    val mutable nodeLabel : labelClass
    val mutable nodeName : domString
    val mutable nodeValue : labeledDomString
    val mutable nodetagName : domString
    val mutable ownerDocument : document
    val mutable parentNode : node' option
    val mutable parentNodeLabel : labelClass
    val mutable prefix : domString
    val mutable previousSibling : node' option
    val mutable previousSiblingLabel : labelClass
    val mutable qualifiedName : domString
    (* val mutable udHandler : userDataHandler *)
    val mutable userData : (domString, domUserData) Hashtbl.t
    method addEventListener : eventType -> eventListener -> bool -> unit
    method appendChild : node' -> bool
    method cloneNode : bool -> node'
    method cloneSubTree : node'
    method cloneWithChildren : node'
    method cloneWithoutChildren : node'
    method dispatchEvent : event -> node' -> bool
    method private existInlist :
      (eventListener * bool) list -> eventListener -> bool -> bool
    method getChildList : childNodeList
    method getEventHandlers : eventType -> (eventListener*bool) list
    method getNextSibling : (node' option*labelClass)
    method getNodeTagName : (domString*labelClass)
    method getPreviousSibling : (node' option*labelClass)
    method getUserData : domString -> domUserData * labelClass
    method getbaseURI : domString * labelClass
    method getDocLabel : labelClass
    method getEventListenerMap :
      (eventType, (eventListener * bool) list) Hashtbl.t
    method getFirstChild : (node' option * labelClass)
    method getFirstChildLabel : labelClass
    method getLastChild : (node' option * labelClass)
    method getLastChildLabel : labelClass
    method getLocalName : domString * labelClass
    method getnType : (nodetype*labelClass)
    method getNamespaceURI : domString * labelClass
    method getNextSiblingLabel : labelClass
    method getNodeLabel : labelClass
    method getNodeName : domString * labelClass
    method getNodeValue : labeledDomString
    method getOwnerDocument : (document*labelClass)
    method getParentNode : (node' option * labelClass)
    method getParentNodeLabel : labelClass
    method getprefix : domString * labelClass
    method getPreviousSiblingLabel : labelClass
    method getqualifiedName : domString * labelClass
    method hasAttributes : bool
    method hasChildNodes : bool * labelClass
    method insertBefore : node' -> node' option -> bool
    method isEqualNode : node' -> bool
    method private isParent : node' -> bool
    method isSameNode : node' -> bool 
    method mergeEmptyAndAdjacentTextNodes : unit
    method normalize : unit -> unit
    method removeChild : node' -> bool
    method removeEventListener : eventType -> eventListener -> bool -> unit
    method private removeFromList :
      (eventListener * bool) list ->
      eventListener -> bool -> (eventListener * bool) list
    method private removeNodeWithChildren : node' -> unit
    method replaceChild : node' -> node' -> bool
    method setContainerNode : bool -> unit
    method setNextSibling : (node' option) -> unit
    method setNodeTagName : (domString*labelClass) -> unit
    method setPreviousSibling : (node' option) -> unit
    method setUserData : domString -> domUserData -> unit
    method setbaseURI : (domString*labelClass) -> unit
    method setDocLabel : labelClass -> unit
    method setEventListenerMap :
      (eventType, (eventListener * bool) list) Hashtbl.t -> unit
    method setFirstChild : node' option -> unit
    method setFirstChildLabel : labelClass -> unit
    method setLastChild : (node' option) -> unit
    method setLastChildLabel : labelClass -> unit
    method setlocalName : (domString*labelClass) -> unit
    method setnType : (nodetype*labelClass) -> unit
    method setNamespaceURI : domString -> unit
    method setNextSiblingLabel : labelClass -> unit
    method setNodeLabel : labelClass -> unit
    method setNodeName : (domString*labelClass) -> unit
    method setNodeValue : labeledDomString -> unit
    method setOwnerDocument : (document) -> unit
    method setParentNode : (node' option) -> unit
    method setParentNodeLabel : labelClass -> unit
    method setprefix : (domString*labelClass) -> unit
    method setPreviousSiblingLabel : labelClass -> unit
    method setqualifiedName : (domString*labelClass) -> unit
  end
and processingInstruction' :
  object
    inherit node'
    val mutable baseURI : domString
    val mutable childNodes : childNodeList
    val mutable data : string
    val mutable docLabel : labelClass
    val mutable eventListenerMap :
      (eventType, (eventListener * bool) list) Hashtbl.t
    val mutable firstChild : node' option
    val mutable firstChildLabel : labelClass
    val mutable isContainerNode : bool
    val mutable lastChild : node' option
    val mutable lastChildLabel : labelClass
    val mutable localName : domString
    val mutable mattr : attrnamedNodeMap
    val mutable nType : nodetype
    val mutable namespaceURI : domString
    val mutable nextSibling : node' option
    val mutable nextSiblingLabel : labelClass
    val mutable nodeLabel : labelClass
    val mutable nodeName : domString
    val mutable nodeValue : labeledDomString
    val mutable nodetagName : domString
    val mutable ownerDocument : document
    val mutable parentNode : node' option
    val mutable parentNodeLabel : labelClass
    val mutable prefix : domString
    val mutable previousSibling : node' option
    val mutable previousSiblingLabel : labelClass
    val mutable qualifiedName : domString
    val mutable target : string
    (* val mutable udHandler : userDataHandler *)
    val mutable userData : (domString, domUserData) Hashtbl.t
    method addEventListener : eventType -> eventListener -> bool -> unit
    method appendChild : node' -> bool
    method cloneNode : bool -> node'
    method cloneSubTree : node'
    method cloneWithChildren : node'
    method cloneWithoutChildren : node'
    method dispatchEvent : event -> node' -> bool
    method private existInlist :
      (eventListener * bool) list -> eventListener -> bool -> bool
    method getChildList : childNodeList
    method getEventHandlers : eventType -> (eventListener*bool) list
    method getNextSibling : (node' option*labelClass)
    method getNodeTagName : (domString*labelClass)
    method getPreviousSibling : (node' option*labelClass)
    method getUserData : domString -> domUserData * labelClass
    method getbaseURI : domString * labelClass
    method getData : string * labelClass
    method getDocLabel : labelClass
    method getEventListenerMap :
      (eventType, (eventListener * bool) list) Hashtbl.t
    method getFirstChild : (node' option * labelClass)
    method getFirstChildLabel : labelClass
    method getLastChild : (node' option * labelClass)
    method getLastChildLabel : labelClass
    method getLocalName : domString * labelClass
    method getnType : (nodetype*labelClass)
    method getNamespaceURI : domString * labelClass
    method getNextSiblingLabel : labelClass
    method getNodeLabel : labelClass
    method getNodeName : domString * labelClass
    method getNodeValue : labeledDomString
    method getOwnerDocument : (document*labelClass)
    method getParentNode : (node' option * labelClass)
    method getParentNodeLabel : labelClass
    method getprefix : domString * labelClass
    method getPreviousSiblingLabel : labelClass
    method getqualifiedName : domString * labelClass
    method getTarget : string * labelClass
    method hasAttributes : bool
    method hasChildNodes : bool * labelClass
    method insertBefore : node' -> node' option -> bool
    method isEqualNode : node' -> bool
    method private isParent : node' -> bool
    method isSameNode : node' -> bool 
    method mergeEmptyAndAdjacentTextNodes : unit
    method normalize : unit -> unit
    method removeChild : node' -> bool
    method removeEventListener : eventType -> eventListener -> bool -> unit
    method private removeFromList :
      (eventListener * bool) list ->
      eventListener -> bool -> (eventListener * bool) list
    method private removeNodeWithChildren : node' -> unit
    method replaceChild : node' -> node' -> bool
    method setContainerNode : bool -> unit
    method setNextSibling : (node' option) -> unit
    method setNodeTagName : (domString*labelClass) -> unit
    method setPreviousSibling : (node' option) -> unit
    method setUserData : domString -> domUserData -> unit
    method setbaseURI : (domString*labelClass) -> unit
    method setDocLabel : labelClass -> unit
    method setData : (domString*labelClass) -> unit
    method setEventListenerMap :
      (eventType, (eventListener * bool) list) Hashtbl.t -> unit
    method setFirstChild : node' option -> unit
    method setFirstChildLabel : labelClass -> unit
    method setLastChild : (node' option) -> unit
    method setLastChildLabel : labelClass -> unit
    method setlocalName : (domString*labelClass) -> unit
    method setnType : (nodetype*labelClass) -> unit
    method setNamespaceURI : domString -> unit
    method setNextSiblingLabel : labelClass -> unit
    method setNodeLabel : labelClass -> unit
    method setNodeName : (domString*labelClass) -> unit
    method setNodeValue : labeledDomString -> unit
    method setOwnerDocument : (document) -> unit
    method setParentNode : (node' option) -> unit
    method setParentNodeLabel : labelClass -> unit
    method setprefix : (domString*labelClass) -> unit
    method setPreviousSiblingLabel : labelClass -> unit
    method setqualifiedName : (domString*labelClass) -> unit
    method setTarget : (domString*labelClass) -> unit
  end
and notation :
  object
    val mutable baseURI : domString
    val mutable childNodes : childNodeList
    val mutable docLabel : labelClass
    val mutable eventListenerMap :
      (eventType, (eventListener * bool) list) Hashtbl.t
    val mutable firstChild : node' option
    val mutable firstChildLabel : labelClass
    val mutable isContainerNode : bool
    val mutable lastChild : node' option
    val mutable lastChildLabel : labelClass
    val mutable localName : domString
    val mutable mattr : attrnamedNodeMap
    val mutable nType : nodetype
    val mutable namespaceURI : domString
    val mutable nextSibling : node' option
    val mutable nextSiblingLabel : labelClass
    val mutable nodeLabel : labelClass
    val mutable nodeName : domString
    val mutable nodeValue : labeledDomString
    val mutable nodetagName : domString
    val mutable ownerDocument : document
    val mutable parentNode : node' option
    val mutable parentNodeLabel : labelClass
    val mutable prefix : domString
    val mutable previousSibling : node' option
    val mutable previousSiblingLabel : labelClass
    val mutable publicID : string
    val mutable qualifiedName : domString
    val mutable systemID : string
    (* val mutable udHandler : userDataHandler *)
    val mutable userData : (domString, domUserData) Hashtbl.t
    method addEventListener : eventType -> eventListener -> bool -> unit
    method appendChild : node' -> bool
    method cloneNode : bool -> node'
    method cloneSubTree : node'
    method cloneWithChildren : node'
    method cloneWithoutChildren : node'
    method dispatchEvent : event -> node' -> bool
    method private existInlist :
      (eventListener * bool) list -> eventListener -> bool -> bool
    method getChildList : childNodeList
    method getEventHandlers : eventType -> (eventListener*bool) list
    method getNextSibling : (node' option*labelClass)
    method getNodeTagName : (domString*labelClass)
    method getPreviousSibling : (node' option*labelClass)
    method getUserData : domString -> domUserData * labelClass
    method getbaseURI : domString * labelClass
    method getDocLabel : labelClass
    method getEventListenerMap :
      (eventType, (eventListener * bool) list) Hashtbl.t
    method getFirstChild : (node' option * labelClass)
    method getFirstChildLabel : labelClass
    method getLastChild : (node' option * labelClass)
    method getLastChildLabel : labelClass
    method getLocalName : domString * labelClass
    method getnType : (nodetype*labelClass)
    method getNamespaceURI : domString * labelClass
    method getNextSiblingLabel : labelClass
    method getNodeLabel : labelClass
    method getNodeName : domString * labelClass
    method getNodeValue : labeledDomString
    method getOwnerDocument : (document*labelClass)
    method getParentNode : (node' option * labelClass)
    method getParentNodeLabel : labelClass
    method getprefix : domString * labelClass
    method getPreviousSiblingLabel : labelClass
    method getpublicId : domString * labelClass
    method getqualifiedName : domString * labelClass
    method getsystemId : domString * labelClass
    method hasAttributes : bool
    method hasChildNodes : bool * labelClass
    method insertBefore : node' -> node' option -> bool
    method isEqualNode : node' -> bool
    method private isParent : node' -> bool
    method isSameNode : node' -> bool 
    method mergeEmptyAndAdjacentTextNodes : unit
    method normalize : unit -> unit
    method removeChild : node' -> bool
    method removeEventListener : eventType -> eventListener -> bool -> unit
    method private removeFromList :
      (eventListener * bool) list ->
      eventListener -> bool -> (eventListener * bool) list
    method private removeNodeWithChildren : node' -> unit
    method replaceChild : node' -> node' -> bool
    method setContainerNode : bool -> unit
    method setNextSibling : (node' option) -> unit
    method setNodeTagName : (domString*labelClass) -> unit
    method setPreviousSibling : (node' option) -> unit
    method setUserData : domString -> domUserData -> unit
    method setbaseURI : (domString*labelClass) -> unit
    method setDocLabel : labelClass -> unit
    method setEventListenerMap :
      (eventType, (eventListener * bool) list) Hashtbl.t -> unit
    method setFirstChild : node' option -> unit
    method setFirstChildLabel : labelClass -> unit
    method setLastChild : (node' option) -> unit
    method setLastChildLabel : labelClass -> unit
    method setlocalName : (domString*labelClass) -> unit
    method setnType : (nodetype*labelClass) -> unit
    method setNamespaceURI : domString -> unit
    method setNextSiblingLabel : labelClass -> unit
    method setNodeLabel : labelClass -> unit
    method setNodeName : (domString*labelClass) -> unit
    method setNodeValue : labeledDomString -> unit
    method setOwnerDocument : (document) -> unit
    method setParentNode : (node' option) -> unit
    method setParentNodeLabel : labelClass -> unit
    method setprefix : (domString*labelClass) -> unit
    method setPreviousSiblingLabel : labelClass -> unit
    method setpublicId : (domString*labelClass) -> unit
    method setqualifiedName : (domString*labelClass) -> unit
    method setsystemId : (domString*labelClass) -> unit
  end
class domError :
  object
    val mutable errorLabel : labelClass
    val mutable name : string
    method getname : string * labelClass
    method setname : string -> unit
  end
class nameList :
  object
    val mutable nList : (domString * domString) list
    method contains : domString -> bool
    method containsNS : domString -> domString -> bool
    method private containsName :
      domString -> (domString * domString) list -> bool
    method private containsTuple :
      domString -> domString -> (domString * domString) list -> bool
    method private getTupleList : (domString * domString) list
    method getname : int -> domString
    method getNamespaceURI : int -> domString
    method private insertTuple :
      domString * domString -> (domString * domString) list
  end
