open DomEventsInterface;;
(*************************************************************************)
class event = object(self)
  val mutable phase = START_PHASE;
  val mutable bubbles = false;
  val mutable cancelable = false; (* if default action can be prevented *)
  val mutable propagationStopped = false;
  val mutable defaultPrevented = false;
  val mutable dispatched = false;
  val mutable stopImmediatePropagation = false;
  val mutable timestamp = 0.0

  method getPhase = phase
  method setPhase (p:eventPhase) = phase <- p

  val mutable eventLabel:labelClass = new labelClass
  method getEventLabel = eventLabel
  method setEventLabel (l:labelClass) = eventLabel <- l

  val mutable target:eventTarget = new eventTarget
  method getTarget = target
  method setTarget (t:eventTarget) = target <- t

  val mutable currentTarget:eventTarget = new eventTarget
  method getCurrentTarget = currentTarget 
  method setCurrentTarget (t:eventTarget) = currentTarget <- t

  method getTimestamp = timestamp
  method setTimestamp (t:float) = timestamp <- t

  method canBubble = bubbles

  val mutable evnType:eventType = BLANK_EVENT;
  method getEventType = (evnType, eventLabel)
  method setEventType (e:eventType) = evnType <- e;
				     
  method initEvent (etype:eventType) (canbub:bool) (cancel:bool) =
    if dispatched = true then 
      ()
    else
      (
	propagationStopped <- false;
	stopImmediatePropagation <- false;
	defaultPrevented <- false;
	bubbles <- canbub; 
	cancelable <- cancel;
	evnType <- etype; 
	();
      )

  method stopPropagation = propagationStopped <- true

  method getStopPropagation = propagationStopped

  method stopImmediatePropagation () = 
    stopImmediatePropagation <- true

  method getStopImmediatePropagation = stopImmediatePropagation

  method defaultPrevented = defaultPrevented

  method preventDefault = 
    if cancelable = true then
      defaultPrevented <- true
    else
      defaultPrevented <- defaultPrevented

end 
(*************************************************************************)
and uiEvent = object(self)
inherit event
  val mutable view:string = "";
  val mutable detail:int = 0;
  method initUIEvent (etype:eventType) =
    if dispatched = true then 
      ()
    else
      (
	propagationStopped <- false;
	stopImmediatePropagation <- false;
	defaultPrevented <- false;
	bubbles <- false; 
	cancelable <- false;
	evnType <- etype; 
	view <- "window";
	detail <- 0;
	();
      )	
end
(*************************************************************************)
and focusEvent = object(self)
inherit uiEvent
  val mutable relatedTarget:eventTarget = new eventTarget
  method initFocusEvent (etype:eventType) (rTarget:eventTarget) =
    if dispatched = true then 
      ()
    else
      (
	propagationStopped <- false;
	stopImmediatePropagation <- false;
	defaultPrevented <- false;
	bubbles <- false; 
	cancelable <- false;
	evnType <- etype; 
	view <- "window";
	detail <- 0;
	relatedTarget <- rTarget;
	();
      )	
end
(*************************************************************************)
and sharedKeyboardMouseEvent = object(self)
inherit uiEvent
   val mutable ctrlKey:bool = false;
   val mutable shiftKey:bool = false;
   val mutable altKey:bool = false;
   val mutable metaKey:bool = false;
   val mutable keyModifierStateAltGraph:bool = false;
   val mutable keyModifierStateCapsLock:bool = false;
   val mutable keyModifierStateFn:bool = false;
   val mutable keyModifierStateFnLock:bool = false;
   val mutable keyModifierStateHyper:bool = false;
   val mutable keyModifierStateNumLock:bool = false;
   val mutable keyModifierStateOS:bool = false;
   val mutable keyModifierStateScrollLock:bool = false;
   val mutable keyModifierStateSuper:bool = false;
   val mutable keyModifierStateSymbol:bool = false;
   val mutable keyModifierStateSymbolLock:bool = false;
   method initSharedKeyboardMouseEvent (etype:eventType) (keyArg:domString)  =
    if dispatched = true then 
      ()
    else
      (
	propagationStopped <- false;
	stopImmediatePropagation <- false;
	defaultPrevented <- false;
	bubbles <- false; 
	cancelable <- false;
	evnType <- etype; 
	view <- "window";
	detail <- 0;
	if keyArg = "Accel"
	then ctrlKey <- true
	else if keyArg = "Alt" 
	then altKey <- true
	else if keyArg = "AltGraph" 
	then keyModifierStateAltGraph <- true
	else if keyArg = "CapsLock" 
	then keyModifierStateCapsLock <- true
	else if	keyArg = "Control" 
	then ctrlKey <- true
	else if keyArg = "Fn" 
	then keyModifierStateFn <- true
	else if keyArg = "FnLock" 
	then keyModifierStateFnLock <- true
	else if keyArg = "Hyper" 
	then keyModifierStateHyper <- true
	else if keyArg = "Meta" 
	then metaKey <- true
	else if keyArg = "NumLock" 
	then keyModifierStateNumLock <- true
	else if keyArg = "OS" 
	then keyModifierStateOS <- true
	else if keyArg = "ScrollLock"
	then keyModifierStateScrollLock <- true 
	else if	keyArg = "Shift" 
	then shiftKey <- true
	else if keyArg = "Super" 
	then keyModifierStateSuper <- true
	else if keyArg = "Symbol" 
	then keyModifierStateSymbol <- true
	else if keyArg = "SymbolLock"
	then keyModifierStateSymbolLock <- true
      )

   method getModifierState (keyArg:domString) : (bool) = 
     (
     if keyArg = "Accel"
     then ctrlKey
     else if keyArg = "Alt" 
     then altKey 
     else if keyArg = "AltGraph" 
     then keyModifierStateAltGraph 
     else if keyArg = "CapsLock" 
     then keyModifierStateCapsLock 
     else if keyArg = "Control" 
     then ctrlKey 
     else if keyArg = "Fn" 
     then keyModifierStateFn 
     else if keyArg = "FnLock" 
     then keyModifierStateFnLock
     else if keyArg = "Hyper" 
     then keyModifierStateHyper
     else if keyArg = "Meta" 
     then metaKey 
     else if keyArg = "NumLock" 
     then keyModifierStateNumLock 
     else if keyArg = "OS" 
     then keyModifierStateOS 
     else if keyArg = "ScrollLock"
     then keyModifierStateScrollLock 
     else if keyArg = "Shift" 
     then shiftKey 
     else if keyArg = "Super" 
     then keyModifierStateSuper 
     else if keyArg = "Symbol" 
     then keyModifierStateSymbol
     else if keyArg = "SymbolLock"
     then keyModifierStateSymbolLock
     else
       false
     )
end
(*************************************************************************)
and mouseEvent = object(self)
inherit sharedKeyboardMouseEvent as super
  val mutable screenX:int = 0
  val mutable screenY:int = 0
  val mutable clientX:int = 0
  val mutable clientY:int = 0
  val mutable ctlKey:bool = false
  val mutable shiftKey:bool = false
  val mutable altKey:bool = false
  val mutable metaKey:bool = false
  val mutable button:int = 0
  val mutable buttons:int = 0
  val mutable relatedTarget:eventTarget = new eventTarget

  method initMouseEvent (etype:eventType) (keyArg:domString) (d:int) (sx:int) (sy:int) (cx:int) 
			(cy:int) (ak:bool) (ck:bool) (sk:bool) (mk:bool) 
			(b:int) (bs:int) (rTarget:eventTarget) =
    if dispatched = true then 
      ()
    else
      (
	super#initSharedKeyboardMouseEvent etype keyArg;
	propagationStopped <- false;
	stopImmediatePropagation <- false;
	defaultPrevented <- false;
	bubbles <- false; 
	cancelable <- false;
	evnType <- etype; 
	view <- "window";
	detail <- d;
	screenX <- sx;
	screenY <- sy;
	clientX <- cx;
	clientY <- cy;
	ctlKey <- ck;
	altKey <- ak;
	shiftKey <- sk;
	metaKey <- mk;
	button <- b;
	buttons <- bs;
	relatedTarget <- rTarget;
	();
      )	
end
(*************************************************************************)
and wheelEvent = object(self)
inherit mouseEvent
  val mutable deltaX:float = 0.0
  val mutable deltaY:float = 0.0
  val mutable deltaZ:float = 0.0
  val mutable deltaMode:int = 0
  method initWheelEvent (etype:eventType) (sx:int) (sy:int) (cx:int) 
			(cy:int) (ak:bool) (ck:bool) (sk:bool) (mk:bool) 
			(b:int) (bs:int) (dx:float) (dy:float) (dz:float) (dm:int) (rTarget:eventTarget) = 
    if dispatched = true then 
      ()
    else
      (
	propagationStopped <- false;
	stopImmediatePropagation <- false;
	defaultPrevented <- false;
	bubbles <- false; 
	cancelable <- false;
	evnType <- etype; 
	view <- "window";
	detail <- 0;
	screenX <- sx;
	screenY <- sy;
	clientX <- cx;
	clientY <- cy;
	ctlKey <- ck;
	altKey <- ak;
	shiftKey <- sk;
	metaKey <- mk;
	button <- b;
	buttons <- bs;
	relatedTarget <- rTarget;
	deltaX <- dx;
	deltaY <- dy;
	deltaZ <- dz;
	deltaMode <- dm;
	();
      )	
end
(*************************************************************************)
and inputEvent = object(self)
inherit uiEvent
  val mutable data:domString = ""
  val mutable isComposing:bool = false
  method initInputEvent (etype:eventType) (d:domString) (c:bool) = 
    if dispatched = true then 
      ()
    else
      (
	propagationStopped <- false;
	stopImmediatePropagation <- false;
	defaultPrevented <- false;
	bubbles <- false; 
	cancelable <- false;
	evnType <- etype; 
	view <- "window";
	detail <- 0;
	data <- d;
	isComposing <- c;
	();
      )	
end
(*************************************************************************)
and keyboardEvent = object(self)
inherit sharedKeyboardMouseEvent as super
  val mutable key:domString = ""
  val mutable location:int = 0
  val mutable ctlKey:bool = false
  val mutable shiftKey:bool = false
  val mutable altKey:bool = false
  val mutable metaKey:bool = false
  val mutable repeat:bool = false
  val mutable isComposing:bool = false

  method initKeyboardEvent (etype:eventType) (k:domString) (l:int) (ak:bool) (ck:bool) (sk:bool) (mk:bool) 
			(r:bool) (c:bool)  =
    if dispatched = true then 
      ()
    else
      (
	super#initSharedKeyboardMouseEvent etype k;
	propagationStopped <- false;
	stopImmediatePropagation <- false;
	defaultPrevented <- false;
	bubbles <- false; 
	cancelable <- false;
	evnType <- etype; 
	view <- "window";
	detail <- 0;
	key <- k;
	location <- l;
	ctlKey <- ck;
	altKey <- ak;
	shiftKey <- sk;
	metaKey <- mk;
	repeat <- r;
	isComposing <- c;
	();
      )	
end
(*************************************************************************)
and compositionEvent = object(self)
inherit uiEvent
  val mutable data:domString = ""
  method initCompEvent (etype:eventType) (d:domString) =
    if dispatched = true then 
      ()
    else
      (
	propagationStopped <- false;
	stopImmediatePropagation <- false;
	defaultPrevented <- false;
	bubbles <- false; 
	cancelable <- false;
	evnType <- etype; 
	view <- "window";
	detail <- 0;
	data <- d;
	();
      )	
end
(************************************************************)
and eventListener = object(self)
  val mutable useCapture:bool = false
  val mutable evtType:eventType = BLANK_EVENT
  val mutable  f = new jsFunctionObject

  method getEventType = evtType 
  method setEventType (e:eventType) = evtType <- e

  method setCapture (c:bool) = useCapture <- c

  (* e:event is required for checking the phase and other context data*)
  method handleEvent (e:event) = 
    let x=() in
    let oldpc = pcLabel in
    pcLabel#setLabel (pcLabel#join (e#getEventLabel));
    self#invokeJsc;
    pcLabel#setLabel (oldpc);
    ()

  method private invokeJsc = 
    let x = () in
    pcLabel#setLabel (pcLabel#join (f#getfunctionObjectLabel));
    ()

end
(*************************************************************************)
and eventTarget = object(self)
  val mutable eventTargetLabel = new labelClass
  val mutable eventListenerMap = Hashtbl.create 1
  method getEventListenerMap = eventListenerMap
  method setEventListenerMap (m) = eventListenerMap <- m

  val mutable defaulteventListenerMap = Hashtbl.create 1

  method private existInlist  (elList:(eventListener * bool) list) (listener:eventListener) (useCapture:bool) = 
    List.exists (fun x -> (listener, useCapture) = x) elList

  method addEventListener (evType:eventType) (listener:eventListener) (useCapture:bool)= 
    if nsuCheck eventTargetLabel pcLabel = false then
      (
	hang(); 
	()
      )
    else
      (
      let el = Hashtbl.find eventListenerMap evType in
      if self#existInlist el listener useCapture = false then
	let nel = el@[(listener,useCapture)] in
	Hashtbl.remove eventListenerMap evType;
	Hashtbl.add eventListenerMap (evType) (nel)
      else
	()
      )

  method private removeFromList (elList:(eventListener * bool) list) (listener:eventListener) (useCapture:bool) = 
    List.filter (fun x -> (listener, useCapture) != x) elList

  method removeEventListener (evType:eventType) (listener:eventListener) (useCapture:bool)=
    if nsuCheck eventTargetLabel pcLabel = false then
      (
	hang(); 
	()
      )
    else
      (
	let el = Hashtbl.find eventListenerMap evType in
	if self#existInlist el listener useCapture= true then
	  let nel = self#removeFromList el listener useCapture in
	  Hashtbl.remove eventListenerMap evType;
	  Hashtbl.add eventListenerMap (evType) (nel)
	else
	  ()
      )

  method private getEventPathToDocumentNode (n:node') : (node' list) = 
    let (pN,pnl) = n#getParentNode in
    match pN with
    | None -> []
    | Some pN' -> 
       let rec loop target pPath =
	 let (nt,nl) = target#getnType in
	 if nt = DOCUMENT_NODE then
           pPath@[target]
	 else
           let (p,pl) = target#getParentNode in
	   match p with
	   | None -> pPath@[target]
	   | Some p' -> loop (p') pPath@[target] in
       loop (pN') []

  method getEventHandlers (t:eventType): ((eventListener * bool)list) = 
    Hashtbl.find eventListenerMap t

  method private getPathLabel (o:node') = 
    let rec getPathLabelInternal (p:node' option) (l:labelClass) =
      match p with
      | None -> l
      | Some p' ->
	 let (pp,ppl) = p'#getParentNode in
	 getPathLabelInternal pp (l#join ppl)
    in
    let (op,opl) = o#getParentNode in
    getPathLabelInternal (op) (opl)

  method dispatchEvent (e:event) (n:node') = 
    let ppL = ((self#getPathLabel n)#join e#getEventLabel)#join n#getNodeLabel in
    if nsuCheck ppL pcLabel = false then
      (
	hang(); 
	false
      )
    else
      (
	let bPath = self#getEventPathToDocumentNode n in
	let cPath = List.rev bPath in
	let rec jscCall (eventHandlers:(eventListener * bool) list) (e:event) = 
	  match eventHandlers with
	  | nil -> false
	  | (he,_)::eH' ->
	     let ht = he#getEventType in
	     let (et,el) = e#getEventType in
      	     if ht = et then
      	       (
      		 he#handleEvent e;
      		 true
      	       )
      	     else
      	       jscCall eH' e;
	in 
	let rec fireEventListeners (nodelist:node' list) (e:event) (p:eventPhase) =
	  match nodelist with
	  | nil -> false
	  | h::nodelist' -> 
	     let (et,el) = e#getEventType in
	     let eventHandlers = h#getEventHandlers (et) in
	     if p != DEFAULT_PHASE then
	       (
		 jscCall eventHandlers e;
		 fireEventListeners nodelist' e;
		 true
	       )
	     else
	       (
		 let dHandler = Hashtbl.find defaulteventListenerMap et in
		 dHandler#run ()
	       )
	in
	let capturePhase (e:event) = 
	  if e#getStopPropagation = true then
	    false
	  else
	    (
	      fireEventListeners (cPath) (e) CAPTURE_PHASE;
	      true
	    )
	in
	let targetPhase (e:event) =
	  let (et,el) = e#getEventType in
	  let eventHandlers = self#getEventHandlers (et) in
	  if e#getStopPropagation = true then
	    false
	  else
	    (
	      jscCall eventHandlers e;
	      true
	    )
	in
	let bubblePhase (e:event) =
	  if e#getStopPropagation = true || e#canBubble = false then
	    false
	  else
	    (
	      fireEventListeners (bPath) (e) BUBBLE_PHASE;
	      true
	    )
	in
	let defaultPhase (e:event) =
	  if e#defaultPrevented = true then
	    false
	  else if e#canBubble = true then
	    ( fireEventListeners (bPath) (e) DEFAULT_PHASE; true )
	  else
	    ( fireEventListeners ([n]) (e) DEFAULT_PHASE;true )
	in
	capturePhase e;
	targetPhase e;
	bubblePhase e;
	defaultPhase e
      )
end
(*************************************************************************)
and virtual newnodeList = object(self)
   method virtual item: int -> (node option * labelClass)
   method virtual length: (int * labelClass)
end
(*************************************************************************)
and childNodeList (owner:node') = object (self)
inherit newnodeList
   val ownerNode = owner
   val mutable listLabel = new labelClass

   method item (index:int) = 
     let rec itemInternal (i:int) (currentNode:node' option) =
       if i = index then
	 (currentNode, listLabel)
       else
	 match currentNode with
	 | None -> 
	    itemInternal (i+1) (None)
	 | Some currentNode' -> 
	    let (nsn, nsl) = currentNode'#getNextSibling in
	    listLabel <- listLabel#join nsl; itemInternal (i+1) (nsn)
     in
     let (fcn,fcl) = ownerNode#getFirstChild in
     listLabel <- fcl;
     itemInternal 0 fcn

   method length = 
     let rec lengthInternal (len:int) (currentNode:node' option) =
       match currentNode with
       | None -> (len, listLabel)
       | Some currentNode' -> 
	  let (ch,chl) = currentNode'#getNextSibling in
	  match ch with
	  | None -> listLabel <- listLabel#join chl; (len, listLabel)
	  | Some ch' -> listLabel <- listLabel#join chl; lengthInternal (len+1) (ch)
     in
     let (fcn,fcl) = ownerNode#getFirstChild in
     listLabel <- fcl;
     lengthInternal (0) fcn
end
(*************************************************************************)
and tagNodeList (root:node') (tag:domString) = object (self)
inherit newnodeList
   val tName = tag
   val rootNode = root
   val mutable listLabel = new labelClass
		 
   method item (index:int) =
     let rec itemInternal (currentNode:node' option) (nList: node list): node list =
       match currentNode with
       | None -> nList
       | Some currentNode' ->
	  let (fcn,fcl) = currentNode'#getFirstChild in
	  match fcn with
	  | None -> 
	     listLabel <- listLabel#join fcl;
	     let (sib,sibl) = currentNode'#getNextSibling in
	     match sib with
	     | None -> listLabel <- listLabel#join sibl; 
		       nList
	     | Some sib' -> 
		let (tn, tl) = sib'#getNodeTagName in
	  	if tName = tn then
		  (
		  listLabel <- listLabel#join sibl;
	  	  itemInternal (sib) (nList@[sib']) 
		  )
	  	else
		  (
		  listLabel <- listLabel#join sibl;
	  	  itemInternal (sib) (nList)
		  )
	  | Some f'->
	     let (tn, tl) = f'#getNodeTagName in
	     if tName = tn then
	       (
	       listLabel <- listLabel#join tl;
	       let nl = itemInternal (fcn) (nList@[f']) in 
	       itemInternal (fcn) (nl)
	       )
	     else
	       (
	       listLabel <- listLabel#join tl;
	       let (fcn,fcl) = f'#getFirstChild in
	       listLabel <- listLabel#join fcl;
	       let nl = itemInternal (fcn) (nList) in
	       let (nsn, nsl) = f'#getNextSibling in
	       listLabel <- listLabel#join nsl;
	       itemInternal (nsn) (nl)
	       )
     in
     let rec nthElement (l:node' list) (n:int) (i:int) = 
       match l with
       | [] -> None
       | h::t -> if i = n then
		   Some h
		 else
		   nthElement t n (i+1)
     in
     let _ = listLabel <- new labelClass in
     let rl = itemInternal (Some rootNode) ([]) in
     (nthElement rl index 0, listLabel)

   method length =
     (* let listLabel = new labelClass in *)
     let rec lengthInternal (currentNode:node' option) (nList: node list): node list =
       match currentNode with
       | None -> nList
       | Some currentNode' ->
	  let (fcn, fcl) = currentNode'#getFirstChild in
	  listLabel <- listLabel#join fcl;
	  match fcn with
	  | None -> 
	     let (sib,sibl) = currentNode'#getNextSibling in
	     listLabel <- listLabel#join sibl;
	     match sib with
	     | None -> nList
	     | Some sib' -> 
		let (tn, tl) = sib'#getNodeTagName in
	  	if tName = tn then
	  	  lengthInternal (sib) (nList@[sib']) 
	  	else
	  	  lengthInternal (sib) (nList)
	     | Some f'->
		let (tn, tl) = f'#getNodeTagName in
		listLabel <- listLabel#join tl;
		if tName = tn then
		  let nl = lengthInternal (fcn) (nList@[f']) in 
		  lengthInternal (fcn) (nl)
		else
		  let (fcn,fcl) = f'#getFirstChild in
		  listLabel <- listLabel#join fcl;
		  let nl = lengthInternal (fcn) (nList) in
		  let (nsn, nsl) = f'#getNextSibling in
		  listLabel <- listLabel#join nsl;
		  lengthInternal (nsn) (nl)
     in
     let _ = listLabel <- new labelClass in
     (List.length (lengthInternal (Some rootNode) ([])), listLabel)
end
(*************************************************************************)
and nsNodeList (root:node') (namespaceURI:domString) (localName:domString) = object (self)
inherit newnodeList
   val nsName = namespaceURI
   val lName = localName
   val rootNode = root
   val mutable listLabel = new labelClass
	 
   method item (index:int) = 
     let rec itemInternal (currentNode:node' option) (nList: node list): node list =
       match currentNode with
       | None -> nList
       | Some currentNode' ->
	  let (fcn,fcl) = currentNode'#getFirstChild in
	  listLabel <- listLabel#join fcl;
	  match fcn with
	  | None -> 
	     let (sib,sibl) = currentNode'#getNextSibling in
	     listLabel <- listLabel#join sibl;
	     match sib with
	     | None -> nList
	     | Some sib' ->
		let (slName, _) = sib'#getLocalName in
		let  (snsName, _) = sib'#getNamespaceURI in
	  	if lName = slName && nsName = snsName then
	  	  itemInternal (sib) (nList@[sib']) 
	  	else
	  	  itemInternal (sib) (nList)
	  | Some f'->
	     let (slName, sll) = f'#getLocalName in
	     let  (snsName, snl) = f'#getNamespaceURI in
	     listLabel <- listLabel#join sll;
	     listLabel <- listLabel#join snl;
	     if lName = slName && nsName = snsName then
	       let nl = itemInternal (fcn) (nList@[f']) in 
	       itemInternal (fcn) (nl)
	     else
	       let (fcn,fcl) = f'#getFirstChild in
	       listLabel <- listLabel#join fcl;
	       let nl = itemInternal (fcn) (nList) in
	       let (nsn, nsl) = f'#getNextSibling in
	       listLabel <- listLabel#join nsl;
	       itemInternal (nsn) (nl)
     in
     let rec nthElement (l:node' list) (n:int) (i:int) = 
       match l with
       | [] -> None
       | h::t -> if i = n then
		   Some h
		 else
		   nthElement t n (i+1)
     in
     let (fcn,fcl) = rootNode#getFirstChild in
     let rl = itemInternal (fcn) ([]) in
     let _ = listLabel <- fcl in
     (nthElement rl index 0, listLabel)

   method length = 
     let rec lengthInternal (currentNode:node' option) (nList: node list): node list =
       match currentNode with
       | None -> nList
       | Some currentNode' ->
	  let (fcn,fcl) = currentNode'#getFirstChild in
	  match fcn with
	  | None -> 
	     let (sib,sibl) = currentNode'#getNextSibling in
	     match sib with
	     | None -> nList
	     | Some sib' -> 
		let (slName, _) = sib'#getLocalName in
		let  (snsName, _) = sib'#getNamespaceURI in
	  	if lName = slName && nsName = snsName then
	  	  lengthInternal (sib) (nList@[sib']) 
	  	else
	  	  lengthInternal (sib) (nList)
	     | Some f'->
		let (slName, _) = f'#getLocalName in
		let  (snsName, _) = f'#getNamespaceURI in
		if lName = slName && nsName = snsName then
		  let nl = lengthInternal (fcn) (nList@[f']) in 
		  lengthInternal (fcn) (nl)
		else
		  let (fcn,fcl) = f'#getFirstChild in
		  let nl = lengthInternal (fcn) (nList) in
		  let (nsn, nsl) = f'#getNextSibling in
		  lengthInternal (nsn) (nl)
     in
     let (fcn,fcl) = rootNode#getFirstChild in
     let _ = listLabel <- fcl in
     (List.length (lengthInternal (fcn) ([])), listLabel)
end
(********************************************************************************)
and node:node' = object (self)
inherit eventTarget
  val mutable nodeName:domString = ""
  val mutable nodeNameLabel: labelClass = pcLabel
  method getNodeName = (nodeName, nodeNameLabel)
  method setNodeName ((n,l)) = 
    (* nodeName <- n *)
    if nsuCheck nodeNameLabel pcLabel = false then
      (
	hang(); 
	()
      )
    else
      (
	nodeName <- n;
	nodeNameLabel <- pcLabel#join l;
	()
      )


  val mutable nodeValue:labeledDomString = ("", pcLabel)
  method getNodeValue = nodeValue
  method setNodeValue (v:labeledDomString) = 
    let (_,l) = nodeValue in
    if nsuCheck l pcLabel = false then
      (
	hang(); 
	()
      )
    else
      (
	let (vts, lts) = v in	
	nodeValue <- (vts, pcLabel#join lts);
	()
      )

  val mutable nType:nodetype = BLANK_NODE
  val mutable nTypeLabel: labelClass = pcLabel
  method getnType = (nType, nTypeLabel)
  method setnType ((t,l):(nodetype*labelClass)) = 
    if nsuCheck nTypeLabel pcLabel = false then
      (
	hang(); 
	()
      )
    else
      (
	nType <- t;
	nTypeLabel <- pcLabel#join l;
	()
      )

  val mutable isContainerNode = false

  val mutable docLabel:labelClass = new labelClass
  method getDocLabel = docLabel
  method setDocLabel (l:labelClass) = docLabel <- l

  val mutable ownerDocument:document = new document
  method getOwnerDocument = (ownerDocument, docLabel)
  method setOwnerDocument (d :document) = 
    if nsuCheck docLabel pcLabel = false then
      (
	hang(); 
	()
      )
    else
      (
	ownerDocument <- d;
	docLabel <- pcLabel;
	()
      )

  val mutable namespaceURI:domString = ""
  val mutable namespaceURIlabel = pcLabel
  method getNamespaceURI = (namespaceURI, namespaceURIlabel)
  method setNamespaceURI (u:domString) = 
    if nsuCheck (self#getNodeLabel) pcLabel = false  then
      let x=() in
      hang(); x
    else
      namespaceURI <- u;
      self#setNodeLabel (namespaceURIlabel#join pcLabel)

  val mutable nodetagName = ""
  val mutable nodetagNameLabel: labelClass = pcLabel
  method getNodeTagName = (nodetagName, nodetagNameLabel)
  method setNodeTagName ((n,l):(domString*labelClass)) = 
      if nsuCheck l pcLabel = false then
      (
       hang(); 
       ()
      )
      else
      (
	nodetagName <- n;
	nodetagNameLabel <- pcLabel#join l;
	()
      )


  val mutable prefix = ""
  val mutable prefixLabel: labelClass = pcLabel
  method getprefix = (prefix, prefixLabel)
  method setprefix ((u,l):(domString*labelClass)) = 
    if nsuCheck prefixLabel pcLabel = false  then
      let x = () in
      hang(); x
    else
      prefix <- u;
      prefixLabel <- (pcLabel#join l) 

  val mutable localName = "" 
  val mutable localNameLabel: labelClass = pcLabel
  method getLocalName = (localName, localNameLabel)
  method setlocalName ((u,l):(domString*labelClass)) = 
    if nsuCheck localNameLabel pcLabel = false  then
      let x = () in
      hang(); x
    else
      localName <- u;
      localNameLabel <- (pcLabel#join l) 

  val mutable baseURI = ""
  val mutable baseURILabel: labelClass = pcLabel
  method getbaseURI = (baseURI, baseURILabel)
  method setbaseURI ((u,l):(domString*labelClass)) = 
    if nsuCheck baseURILabel pcLabel = false  then
      let x=() in
      hang(); x
    else
      baseURI <- u;
      baseURILabel <-  (l#join pcLabel) 

  val mutable qualifiedName = ""  (* prefix ^ ":" ^ localName *)
  val mutable qualifiedNameLabel: labelClass = pcLabel
  method getqualifiedName = (qualifiedName, qualifiedNameLabel)
  method setqualifiedName ((q,l):(domString*labelClass)) = 
    if nsuCheck qualifiedNameLabel pcLabel = false  then
      let x=() in
      hang(); x
    else
      qualifiedName <- q;
      qualifiedNameLabel <- (l#join pcLabel) 

  val mutable userData = Hashtbl.create 1
  method setUserData (key:domString) (data:domUserData) =
    if nsuCheck (self#getNodeLabel) pcLabel = false  then
      let x=() in
      hang();x
    else
      let r = Hashtbl.find userData key in
      Hashtbl.add userData key data;
      self#setNodeLabel  ((self#getNodeLabel)#join pcLabel);
      r; ()

   method getUserData (key:domString) =
     (Hashtbl.find userData key, self#getNodeLabel)

  val mutable lastChildLabel:labelClass = new labelClass
  method getLastChildLabel = lastChildLabel
  method setLastChildLabel (l:labelClass) = lastChildLabel <- l

  val mutable firstChildLabel:labelClass = new labelClass
  method getFirstChildLabel = firstChildLabel
  method setFirstChildLabel (l:labelClass) = firstChildLabel <- l

  val mutable firstChild:node' option = None 
  method setFirstChild (f:node' option) = 
    if nsuCheck self#getFirstChildLabel pcLabel = false then
      let x=() in hang(); x
    else
      firstChild <- f; firstChildLabel <- pcLabel; ()
  method getFirstChild = (firstChild, firstChildLabel)

  val mutable lastChild:node' option = None
  method setLastChild (l : node' option) = 
    if nsuCheck lastChildLabel pcLabel = false then
      let x=() in hang(); x
    else
      (
      lastChild <- l; 
      lastChildLabel <- pcLabel;
      ()
      )
  method getLastChild = (lastChild, lastChildLabel)

  val mutable nodeLabel:labelClass = new labelClass
  method getNodeLabel = nodeLabel
  method setNodeLabel (l:labelClass) = nodeLabel <- l

  val mutable parentNodeLabel:labelClass = new labelClass
  method getParentNodeLabel = parentNodeLabel
  method setParentNodeLabel (l:labelClass) = parentNodeLabel <- l

  val mutable nextSiblingLabel:labelClass = new labelClass
  method getNextSiblingLabel = nextSiblingLabel
  method setNextSiblingLabel (l:labelClass) = nextSiblingLabel <- l

  val mutable previousSiblingLabel:labelClass = new labelClass
  method getPreviousSiblingLabel = previousSiblingLabel
  method setPreviousSiblingLabel (l:labelClass) = previousSiblingLabel <- l

  val mutable parentNode:node' option = None
  method getParentNode = (parentNode, parentNodeLabel)
  method setParentNode (p: node' option) = 
    if nsuCheck parentNodeLabel pcLabel = false then
      let x=() in hang(); x
    else
      (
      parentNode <- p; 
      parentNodeLabel <- (pcLabel);
      ()
      )

  val mutable childNodes:childNodeList = new childNodeList (new node)
  method getChildList = childNodes

  val mutable nextSibling:node' option = None
  method setNextSibling (n:(node' option)) = 
    if nsuCheck nextSiblingLabel pcLabel = false then
      let x=() in hang(); x
    else
      (
	nextSibling <- n; 
	nextSiblingLabel <- (pcLabel);
	()
      )
  method getNextSibling = (nextSibling, nextSiblingLabel)

  val mutable previousSibling:node' option = None

  method setPreviousSibling (n:node' option) = 
    if nsuCheck previousSiblingLabel pcLabel = false then
      let x=() in hang(); x
    else
      (
	previousSibling <- n; 
	previousSiblingLabel <- (pcLabel);
	()
      )
  method getPreviousSibling = (previousSibling, previousSiblingLabel)

  val mutable mattr = new attrnamedNodeMap (new node)
  method getAttr = mattr
  method getAttribute (name:domString) =
      let a = mattr#getNamedItem name in
      match a with
      | None -> None
      | Some a' -> Some a'#getNodeName


  method hasChildNodes =
  let hasChildNodesInternal =
    let (fcn,fcl) = self#getFirstChild in   
    if fcn = None then
      false
    else
      true
  in
  let r = hasChildNodesInternal in
  (r, pcLabel)

  method private isParent (n1:node') = 
    let (p,pl) = self#getParentNode in
    match p with
    | None -> false
    | Some p' ->
       if p' = n1 then
	 true
       else
	 false

  method private removeNodeWithChildren (n:node') =
    (let (next,nextl) = n#getNextSibling in
    match next with
    | None ->
       let (prev,_) = n#getPreviousSibling in
       match prev with
       | None -> 
	  let (par,parl) = n#getParentNode in
	  match par with
	  | Some p ->
	     p#setFirstChild (None);
	     p#setLastChild (None)
       | Some p ->
	  p#setNextSibling (None);
	  let (par,parl) = n#getParentNode in
	  match par with
	  | Some p ->
	     p#setLastChild (prev)
    | Some next' ->
       let (prev,_) = n#getPreviousSibling in
       match prev with
       | None -> 
	  next'#setPreviousSibling (None);
	  let (par,parl) = n#getParentNode in
	  match par with
	  | Some p ->
	     p#setFirstChild next;
	     p#setFirstChildLabel pcLabel
       | Some p ->
	  p#setNextSibling (next);
	  next'#setPreviousSibling (prev)
    );
    n#setParentNode (None)
			      
  method setContainerNode (v:bool) =
    isContainerNode <- v

  method insertBefore (newChild:node') (refChild:node' option) =
    let canInsert =
      match refChild with 
      | None -> true
      | Some rChild ->
	 if self#isParent rChild then
	   true
	 else
	   false
    in
    let insertLogic (newChild:node') (refChild:node' option) =
      newChild#setNextSibling (refChild);
      newChild#setParentNode (Some (self :> node'));
      true
    in
    match refChild with
    | None -> self#appendChild newChild 
    | Some rChild -> 
       if newChild = rChild then
	 true
       else
	 let (opSib,_) = rChild#getPreviousSibling in
	 match opSib with
	 | None -> self#setFirstChild (Some newChild);
		   rChild#setPreviousSibling (Some newChild);
	    true
	 | Some pSib ->
	    if newChild = pSib then
	      true
	    else
	      let nChild = Some newChild in
	      let (p,pl) = newChild#getParentNode in
	      match p with
	      | None -> 
		 newChild#setPreviousSibling (opSib);
		 rChild#setPreviousSibling (nChild);
		 insertLogic newChild refChild
	      | Some p' -> 
		 p'#removeChild newChild;
		 newChild#setPreviousSibling (opSib);
		 rChild#setPreviousSibling (nChild);
		 insertLogic newChild refChild
			     
  method replaceChild (newChild:node') (oldChild:node') =
  let canReplace (newChild:node') (oldChild:node')= 
    if self#isParent oldChild then
      true
    else
      false
  in
  let replaceLogic (newChild:node') (oldChild:node') =
    let (fcn,fcl) = self#getFirstChild in
    match fcn with
    | None -> false 
    | Some f ->  
       if oldChild = f then
	     let (lc,lcl) = self#getLastChild in
	     match lc with
	     | None -> false
	     | Some l ->  
		let (nn,nnl) = oldChild#getNextSibling in
	        self#removeNodeWithChildren oldChild; 
      		self#insertBefore newChild (nn)
       else
	 let (lc,lcl) = self#getLastChild in
	 match lc with
	 | None -> false 
	 | Some l ->  
	    let (nn,nnl) = oldChild#getNextSibling in
	    self#removeNodeWithChildren oldChild; 
	    self#insertBefore newChild (nn)
  in
  if canReplace newChild oldChild = false then
    false
  else
    let t = replaceLogic newChild oldChild in
    match t with
    | true ->
       let n = (self :> node') in
       newChild#setParentNode (Some n);
       true
    | false -> false

  method removeChild (oldChild:node') =
    let canRemove (oldChild:node') = 
      if self#isParent oldChild then
	true
      else
	false
    in
    let removeLogic (oldChild:node') =
      let (fcn,fcl) = self#getFirstChild in
      match fcn with
      | None -> false
      | Some f ->
	 if oldChild = f then
	   (
    	   let (lc,lcl) = self#getLastChild in
    	   match lc with
    	   | None -> false
    	   | Some l -> 
	      (
    	      if oldChild = l then 
    		(self#setFirstChild None;
	        self#setLastChild (None);
    		true)
	      else
    	        let (nn,nnl) = oldChild#getNextSibling in
    		match nn with
    		| None -> false
    		| Some nn' ->
		     (self#setFirstChild nn;
    		     nn'#setPreviousSibling (None);
    		     true)
	      )
	   )
	 else
    	   let (lc,lcl) = self#getLastChild in
    	   match lc with
    	   | None -> false
    	   | Some l ->
    	      if oldChild = l then 
    	 	let (ps,_) = oldChild#getPreviousSibling in
    	 	match ps with
    	 	| None -> false
    	 	| Some ps' ->
    	 	   (self#setLastChild (ps);
    	 	    ps'#setNextSibling (None);
    	 	    true)
    	      else
    		let (p,_) = oldChild#getPreviousSibling in
		match p with
		| None -> false
		| Some p' -> 
		   let (n,nl) = oldChild#getNextSibling in
		   match n with
		   | None -> false
		   | Some n' ->
		      p'#setNextSibling (n);
		      n'#setPreviousSibling (p);
		      true
    in
    if canRemove oldChild = false then 
      false
    else
      let n = removeLogic oldChild in
      match n with 
      | true ->
         oldChild#setParentNode (None);
	 n
      | false -> false

  method appendChild (newChild:node') =
    let canAppend (newChild:node') = 
      let lcl = self#getLastChildLabel in
      if nsuCheck lcl pcLabel = false then
	false
      else
	true
    in
    let appendToTree (newChild:node') =
      let (lc,lcl) = self#getLastChild in
	match lc with
	| None -> 
	   self#setLastChild (Some newChild);
	   self#setFirstChild (Some newChild);
	   true
	| Some lc' ->
	   (
	     self#setLastChild (Some newChild);
	     lc'#setNextSibling (Some newChild);
	     newChild#setPreviousSibling (lc);
	     true
	   )
    in
    let appendLogic (newChild:node') =
	let (np,npl) = newChild#getParentNode in
	match np with
	| None -> appendToTree newChild
	| Some np' ->
	   np'#removeChild newChild;
	   appendToTree newChild
    in
    if canAppend newChild = false then 
      false
    else
      let t = appendLogic newChild in
      match t with
      |true -> let n = (self :> node') in
	       newChild#setParentNode (Some n);
	       true
      |false -> false

  method cloneSubTree =
    let n = new node in
    (* Copy all fields of the object into n*)
    n#setNodeLabel ((self#getNodeLabel)#join pcLabel); 
    let (fcn,fcl) = self#getFirstChild in
    match fcn with
    | None ->
       let (sib,sibl) = self#getNextSibling in
       match sib with
       | None -> 
	  n#setNextSibling (None);
	  n
       | Some sib' -> 
	  let ssib' = sib'#cloneSubTree in
	  n#setNextSibling (Some ssib');
	  n
    | Some f' ->
       let f'fst = f'#cloneSubTree in
       n#setFirstChild (Some f'fst);
       n

  method cloneWithChildren = 
    let nClone = self#cloneWithoutChildren in
    (* Copy all fields of the object into nclone*)
    (* set parent to null *)
    let (fcn,fcl) = self#getFirstChild in
    match fcn with
    | None -> nClone
    | Some f' -> 
       let x = f'#cloneSubTree in
       nClone#setFirstChild (Some x); 
       nClone

  method cloneWithoutChildren =
    let n = new node in
    (* Copy all fields of the object into n except children*)
    n#setNodeLabel ((self#getNodeLabel)#join pcLabel);
    n

  method cloneNode (deep:bool) =
    if deep = true then 
      self#cloneWithChildren
    else
      self#cloneWithoutChildren

  method mergeEmptyAndAdjacentTextNodes = 
  let joinNodes (s:node') = 
    self#setNodeValue ((fst self#getNodeValue ^ fst s#getNodeValue), (snd self#getNodeValue)#join(snd s#getNodeValue));
    let (n, _) = (s#getNextSibling) in
    self#setNextSibling n;
    s#setNextSibling (None);
    s#setPreviousSibling (None);
  in
  let rec mergeSiblings (s:node') =  
    let (nt,nl) = self#getnType in 
    if (nt = TEXT_NODE || nt = BLANK_NODE) && (nt = TEXT_NODE || nt = BLANK_NODE) then
      let sLabel = s#getNodeLabel in
      if nsuCheck sLabel pcLabel = false then
	joinNodes s;
      self#setNodeLabel ((self#getNodeLabel)#join ((s#getNodeLabel)#join pcLabel)); 
      let (x,xl) = self#getNextSibling in
      let nLabel = self#getNextSiblingLabel in 
      match x with
      | None -> ()
      | Some x' -> 
	 if nsuCheck nLabel pcLabel = false then
	   mergeSiblings x'
  in
  let  (s,sl) = self#getNextSibling in
  match s with 
  | None -> 
     (
       let (fcn,fcl) = self#getFirstChild in
       match fcn with
       | None -> ()
       | Some f' ->  f'#mergeEmptyAndAdjacentTextNodes
     )
  | Some sib ->  mergeSiblings sib

   method normalize () = 
     self#mergeEmptyAndAdjacentTextNodes

   method hasAttributes = 
     let r = mattr#possessAttributes in
     r
  
   method isSameNode (n:node') = 
     if (self :> node') = n  then 
       true
     else
       false
     
   method isEqualNode (n:node') =
     self#getnType = n#getnType && self#getChildList = n#getChildList && 
       self#getNodeName = n#getNodeName && self#getNamespaceURI = n#getNamespaceURI && 
	 self#getprefix = n#getprefix && self#getNodeValue = n#getNodeValue && 
	   self#getLocalName = n#getLocalName && self#getAttr = n#getAttr
end
(*************************************************************************)
and  attr:attr' = object (self) 
inherit node
    val mutable attrName = ""
    val mutable attrNameLabel: labelClass = pcLabel
    method getName = (attrName, attrNameLabel)
    method setName ((n,l):(domString*labelClass))= 
      if nsuCheck attrNameLabel pcLabel= false then
	let x=() in
	hang();x
      else
	let x=() in
	attrName <- n;
	attrNameLabel <- pcLabel#join l; 
	x

    val mutable value:domString = ""
    val mutable valueLabel: labelClass = pcLabel
    method getVal = (value, valueLabel)
    method setVal ((v,l):(domString*labelClass)) = 
      if nsuCheck self#getNodeLabel pcLabel= false then
	let x=() in
	hang();x
      else
	let x=() in
	value <- v;
	valueLabel <- pcLabel#join l;
	x
end
(*************************************************************************)
and element (tagName:domString):element' = object (self)
inherit node'

    method getAttribute (name:domString) =
      let a = mattr#getNamedItem name in
      match a with
      | None -> None
      | Some a' -> Some a'#getNodeName

    method setAttribute (name:domString) (value:domString) = 
      let n = new node in
      n#setNodeName (name,pcLabel);
      n#setNodeValue (value, pcLabel);
      mattr#setNamedItem n; ()
			 
    method removeAttribute (name:domString) =
      mattr#removeNamedItem name; ()

    method getAttributeNode (name:domString) = 
      mattr#getNamedItem name

    method setAttributeNode (newattr:attr) = 
      let toNode (a:attr) = (a :> node')
      in 
      let n = toNode newattr in
      mattr#setNamedItem n

    method removeAttributeNode (oldAttr:attr)= 
      let toNode (a:attr) = (a :> node')
      in 
      let n = toNode oldAttr in
      let (nName, nLabel) = n#getNodeName in
      mattr#removeNamedItem nName

    method toNodeofSelf = (self :> node')

    method getElementsByTagName (name:domString) = 
      let r = new tagNodeList (self#toNodeofSelf) (name) in
      r
		
    method getAttributeNS (namespaceURI:domString) (localName:domString) =
      let a = mattr#getNamedItemNS namespaceURI localName in
      match a with
      | None -> None
      | Some a' -> Some a'#getNodeName

    method setAttributeNS (namespaceURI:domString) (qualifiedName:domString) (value:domString) = 
      let n = new node in
      n#setNamespaceURI namespaceURI;
      n#setqualifiedName (qualifiedName,pcLabel);
      n#setNodeValue (value, pcLabel);
      mattr#setNamedItemNS n; ()

    method removeAttributeNS (namespaceURI:domString) (localName:domString)=
      mattr#removeNamedItemNS namespaceURI localName; ()

    method getAttributeNodeNS (namespaceURI:domString) (localName:domString) = 
      mattr#getNamedItemNS namespaceURI localName

    method setAttributeNodeNS (newattr:attr) = 
      let toNode (a:attr) = (a :> node')
      in 
      let n = toNode newattr in
      mattr#setNamedItemNS n

    method getElementsByTagNameNS (namespaceURI:domString) (localName:domString) = 
      let r = new nsNodeList (self#toNodeofSelf) (namespaceURI) (localName) in
      r

    method hasAttribute (name:domString) = 
      let r = mattr#getNamedItem name in
      match r with
      | None -> false
      | Some _ -> true

end
(*************************************************************************)
and document= object(self)
inherit node'
    val mutable documentElement:element' = new element' ""

    val mutable docType:documentType = new documentType
    val mutable docTypeLabel: labelClass = pcLabel
    method getdocType = (docType, docTypeLabel)
    method setdocType ((d,l):(documentType'*labelClass)) = 
      if nsuCheck l pcLabel = false then
      (
       hang(); 
       ()
      )
      else
      (
	docType <- d;
	docTypeLabel <- pcLabel#join l;
	()
      )

    val mutable implementation:domImplementation = new domImplementation
    val mutable docmentURI:domString = ""

    val idElementMap = Hashtbl.create 1
    val idLabelMap = Hashtbl.create 1
    val mutable idElementMapStructLabel = new labelClass

    method removeFromidElementMap (key:string) =
      let l = try Hashtbl.find idLabelMap key with Not_found -> new labelClass in
        if l#getLabel != 0 then
	  (
	  if nsuCheck l pcLabel = false then
	    ( hang();() )
	  else
            Hashtbl.remove idLabelMap key;
	    Hashtbl.remove idElementMap key)

    method insertIntoidElementMap (key:string) (v:node') =
      let l = try Hashtbl.find idLabelMap key with Not_found -> new labelClass in
      if l#getLabel != 0 then
	(self#removeFromidElementMap key;())
      else
	Hashtbl.add idElementMap key v;
      Hashtbl.add idLabelMap key pcLabel

    method createElement (tagName:labeledDomString) = 
      let (tString, tLabel) = tagName in
      let e = new element' tString in
      e#setNodeLabel (pcLabel#join tLabel);
      e

    method createTextNode (data:labeledDomString):text' =
      let (dString, dLabel) = data in
      let t = new text in
      t#insertData 0 dString;
      t#setNodeLabel (pcLabel#join dLabel);
      t

    method createComment (data:labeledDomString):comment' =
      let (dString, dLabel) = data in
      let t = new comment in
       t#insertData 0 dString;
       t#setNodeLabel (pcLabel#join dLabel);
       t

    method createCDATASection (data:labeledDomString):cdataSection' =
      let (dString, dLabel) = data in
      let t = new cdataSection in
      t#insertData 0 dString;
      t#setNodeLabel (pcLabel#join dLabel);
      t

    method createAttribute (name:labeledDomString):attr' =
      let (nString, nLabel) = name in
      let t = new attr in
      t#setName (nString,pcLabel);
      t#setNodeLabel (pcLabel#join nLabel);
      t

    method createProcessingInstruction (target:domString) (data:labeledDomString):processingInstruction' =
      let (dString, dLabel) = data in
      let t = new processingInstruction in
      t#setData (dString,pcLabel);
      t#setTarget (target,pcLabel);
      t#setNodeLabel (pcLabel#join dLabel);
      t

    method createEntityReference (name:labeledDomString):entityReference' =
      let (nString, nLabel) = name in
      let t = new entityReference in
      t#setNodeName (nString,pcLabel);
      t#setNodeLabel (pcLabel#join nLabel);
      t

    method createDocumentFragment () = 
      let d = new documentFragment' in
      d

    method getElementsByTagName (tagName:labeledDomString) =
      let (nTagName, _) = tagName in
      documentElement#getElementsByTagName nTagName
      
    method getElementById (id:domString):node' = 
      let l = try Hashtbl.find idLabelMap id with Not_found -> new labelClass in
      if (l#getLabel = 0) then
      	let rec elID (n:node') =
      	  let att = n#getAttribute id in
      	  match att with
      	  | Some a' ->
      	  if (fst a') = id then
      	    (Hashtbl.add idElementMap id n;
             Hashtbl.add idLabelMap id l;
	     n)
      	  else
      	    let (fcn,fcl) = n#getFirstChild in
      	    match fcn with
      	    | None ->
      	       (
      		 l#join (n#getFirstChildLabel);
      		 l#join (n#getNextSiblingLabel);
      		 let (next,nextl) = n#getNextSibling in
      		 match next with
      		 | None -> 
		    idElementMapStructLabel <- l;
		    new node' 
      		 | Some next' ->
      		    elID (next')
      	       )
      	    | Some child ->
      	       (
      		 l#join (self#getFirstChildLabel);
      		 elID child
      	       )
      	in
        elID (self#toNode);
      else
      	Hashtbl.find idElementMap id
    
    method getElementByIdLabel (id:domString):labelClass =
      let l = try Hashtbl.find idLabelMap id with Not_found -> new labelClass in
      if (l#getLabel = 0) then
	idElementMapStructLabel 
      else
	l

    method importNode (importedNode:node') (deep:bool) = 
      let (t,l) = importedNode#getnType in
      if t = ATTRIBUTE_NODE then
	let r = importedNode#cloneNode true in
	Some r
      else if t = DOCUMENT_FRAGMENT_NODE then
	let r = importedNode#cloneNode deep in
	Some r   
      else if t = DOCUMENT_NODE then
	None
      else if t = DOCUMENT_TYPE_NODE then
	None
      else if t = ELEMENT_NODE then
	let r = importedNode#cloneNode deep in
	Some r  
      else
	let r = importedNode#cloneNode deep in
	Some r   
		    

    method private removeNode (n:node') =
      (let (next,nextl) = n#getNextSibling in
       match next with
       | None ->
	  let (prev,_) = n#getPreviousSibling in
	  match prev with
	  | None -> 
	     let (par,parl) = n#getParentNode in
	     match par with
	     | Some p ->
		p#setFirstChild (None);
		p#setLastChild (None)
	     | Some p ->
		p#setNextSibling (None);
		let (par,parl) = n#getParentNode in
		match par with
		| Some p ->
		   p#setLastChild (prev)
		| Some next' ->
		   let (prev,_) = n#getPreviousSibling in
		   match prev with
		   | None -> 
		      next'#setPreviousSibling (None);
		      let (par,parl) = n#getParentNode in
		      match par with
		      | Some p ->
			 p#setFirstChild next;
			 p#setFirstChildLabel pcLabel
		      | Some p ->
			 p#setNextSibling (next);
			 next'#setPreviousSibling (prev);
      );
      n#setParentNode (None)

    method adoptNode (sourceNode:node'):node' option=
      let (t,l) = sourceNode#getnType in
      match t with
      | ENTITY_NODE -> None
      | NOTATION_NODE -> None
      | DOCUMENT_NODE -> None
      | DOCUMENT_TYPE_NODE -> None
      | XPATH_NAMESPACE_NODE -> None
      | _ ->
    	 let n = self#importNode sourceNode true in
	 match n with
	 | None -> None
	 | Some n' -> 
	    let (d,_) = sourceNode#getOwnerDocument in
	    n'#setOwnerDocument d;
    	    n'#setEventListenerMap sourceNode#getEventListenerMap;
    	    let (p,pl) = sourceNode#getParentNode in
    	    match p with
    	    | None -> None
    	    | Some p' ->
    	       p'#removeChild sourceNode;
    	       documentElement#appendChild n';
    	       n
   
    method toNode = (self :> node')

    method normalizeDocument () =
      let x = self#toNode in
        x#normalize ()
end
(*************************************************************************)
and domImplementation = object(self)

    method createDocument (namespaceURI:domString) (qualifiedName:domString) (docType:documentType) = 
      let d = new document in
      d#setNamespaceURI namespaceURI;
      d#setqualifiedName (qualifiedName,pcLabel);
      d#setdocType (docType, pcLabel);
      d#setNodeLabel pcLabel;
      d

    method hasFeature (feature:domString) (version:domString) = false
end
(*************************************************************************)
and documentType:documentType' = object(self)
inherit node
    val mutable name = ""
    val mutable nameLabel: labelClass = pcLabel
    method getname = (name, nameLabel)
    method setname ((n,l):(domString*labelClass)) = 
      if nsuCheck l pcLabel = false then
      (
       hang(); 
       ()
      )
      else
      (
	name <- n;
	nameLabel <- pcLabel#join l;
	()
      )

    method private toNode () = (self :> node')

    val mutable entities: entitynamedNodeMap = new entitynamedNodeMap (new node')
    val mutable entitiesLabel: labelClass = pcLabel
    method getentities = (entities, entitiesLabel)
    method setentities ((n,l):(node'*labelClass)) = 
      if nsuCheck entitiesLabel pcLabel = false then
      (
       hang(); 
       ()
      )
      else
      (
	entities <- new entitynamedNodeMap (n);
	entitiesLabel <- pcLabel#join l;
	()
      )

    val mutable notations: notationnamedNodeMap = new notationnamedNodeMap (new node')
    val mutable notationsLabel: labelClass = pcLabel
    method getnotations = (notations, notationsLabel)
    method setnotations ((n,l):(node'*labelClass)) = 
      if nsuCheck notationsLabel pcLabel = false then
      (
       hang(); 
       ()
      )
      else
      (
	notations <- new notationnamedNodeMap (n);
	notationsLabel <- pcLabel#join l;
	()
      )


    val mutable publicId = ""
    val mutable publicIdLabel: labelClass = pcLabel
    method getpublicId = (publicId, publicIdLabel)
    method setpublicId (p,l) = 
      if nsuCheck publicIdLabel pcLabel = false then
      (
       hang(); 
       ()
      )
      else
      (
	publicId <- p;
	publicIdLabel <- pcLabel#join l;
	()
      )

    val mutable systemId = ""
    val mutable systemIdLabel: labelClass = pcLabel
    method getsystemId = (systemId, systemIdLabel)
    method setsystemId (s,l) = 
      if nsuCheck systemIdLabel pcLabel = false then
      (
       hang(); 
       ()
      )
      else
      (
	systemId <- s;
	systemIdLabel <- pcLabel#join l;
	()
      )

    val mutable internalSubset = ""
    val mutable internalSubsetLabel: labelClass = pcLabel
    method getinternalSubset = (internalSubset, internalSubsetLabel)
    method setinternalSubset (i,l) = 
      if nsuCheck internalSubsetLabel pcLabel = false then
      (
       hang(); 
       ()
      )
      else
      (
	internalSubset <- i;
	internalSubsetLabel <- pcLabel#join l;
	()
      )
end
(*************************************************************************)
and documentFragment:documentFragment' = object(self)
inherit node
end
(*************************************************************************)
and characterData = object(self)
inherit node
    val mutable data:domString = ""
    val mutable stringStructLabel:labelClass = new labelClass

    method substringData (offset:int) (count:int) =
      let r = String.sub data offset count in
      (r,(stringStructLabel#join pcLabel)#join self#getNodeLabel)

    method insertData (offset:int) (arg:domString) = 
      if nsuCheck stringStructLabel pcLabel = false then
	let x=() in
	hang();x
      else
	let x=() in
	stringStructLabel <- stringStructLabel#join pcLabel;
	String.blit arg 0 data offset (String.length arg) 

    method appendData (arg:domString) = 
      if nsuCheck stringStructLabel pcLabel = false then
	let x=() in
	hang();x
      else
	let x=() in
	stringStructLabel <- stringStructLabel#join pcLabel;
	String.concat "" [data;arg]; ()

    method deleteData (offset:int) (count:int) =
      if nsuCheck stringStructLabel pcLabel = false then
	let x=() in
	hang();x
      else
	let x=() in
	let l = String.length data in
        let s1 = String.sub data 0 (offset+1) in
	let s2 = String.sub data (offset+count) (l-offset-count) in
	stringStructLabel <- stringStructLabel#join pcLabel;
	data <- String.concat "" [s1;s2];
	x

    method replaceData (offset:int) (count:int) (arg:domString) =
      if nsuCheck stringStructLabel pcLabel = false then
	let x=() in
	hang();x
      else
	let x=() in
	let l = String.length data in
        let s1 = String.sub data 0 (offset+1) in
	let s2 = String.sub data (offset+count) (l-offset-count) in
	stringStructLabel <- stringStructLabel#join pcLabel;
	data <- String.concat "" [s1;arg;s2];
	x
end
(*************************************************************************)
and text = object(self)
inherit characterData

    method toNode (t:text) = (t :> node')
      
    method splitText (offset:int) = 
      let l = String.length data in
      let s1 = String.sub data 0 (offset) in
      let s2 = String.sub data (offset) (l-1) in
      let newText = new text in
      let (sibling,siblingl) = self#getNextSibling in
      self#replaceData 0 offset s1;
      newText#insertData 0 s2;
      self#insertBefore (self#toNode newText) sibling       
end
(*************************************************************************)
and comment:comment' = object(self)
inherit characterData'
end
(*************************************************************************)
and cdataSection:cdataSection' = object(self)
inherit characterData'
end
(*************************************************************************)
and newnamedNodeMap = object (self)
end
(*************************************************************************)
and attrnamedNodeMap (root:node') = object (self)
inherit newnamedNodeMap
  val rootNode = root
  method possessAttributes =
    let rec possessAttributesInternal (currentNode:node' option) = 
      match currentNode with
      | None -> false
      | Some currentNode' -> 
	 let (nName,nLabel) = currentNode'#getNodeName in
	 let (nt,nl) = currentNode'#getnType in
	 if nt = ATTRIBUTE_NODE then 
	   true
	 else
	   let (nsn, nsl) = currentNode'#getNextSibling in
	   possessAttributesInternal (nsn)
    in
    let (fcn,fcl) = rootNode#getFirstChild in
    possessAttributesInternal fcn 

  method getNamedItem (name:domString) = 
    let rec getNamedItemInternal (currentNode:node' option) = 
      match currentNode with
      | None -> None
      | Some currentNode' -> 
	 let (nName,nLabel) = currentNode'#getNodeName in
	 let (nt,nl) = currentNode'#getnType in
	 if name =  nName && nt = ATTRIBUTE_NODE then 
	   currentNode
	 else
	   let (nsn, nsl) = currentNode'#getNextSibling in
	   getNamedItemInternal (nsn)
    in
    let (fcn,fcl) = rootNode#getFirstChild in
    getNamedItemInternal fcn

  method setNamedItem (ard:node') =
    let rec setNamedItemInternal (currentNode:node' option) = 
      match currentNode with
      | None -> None
      | Some currentNode' -> 
	 let (nName,nLabel) = currentNode'#getNodeName in
	 let (aName,aLabel) = ard#getNodeName in
	 let (nt,nl) = currentNode'#getnType in
	 if  nName = aName &&  nt = ATTRIBUTE_NODE then 
	   let (p,pl) = currentNode'#getParentNode in
	   match p with
	   | None -> None
	   | Some p' ->
	      p'#replaceChild currentNode' ard;
	      currentNode
	 else
	   let (nsn, nsl) = currentNode'#getNextSibling in
	   setNamedItemInternal (nsn)
    in
    let (fcn,fcl) = rootNode#getFirstChild in
    setNamedItemInternal fcn

  method removeNamedItem (name:domString) =
    let rec removeNamedItemInternal (currentNode:node' option) = 
      match currentNode with
      | None -> None
      | Some currentNode' -> 
	 let (nName,nLabel) = currentNode'#getNodeName in
	 let (nt,nl) = currentNode'#getnType in
	 if  nName = name &&  nt = ATTRIBUTE_NODE then 
	   let (p,pl) = currentNode'#getParentNode in
	   match p with
	   | None -> None
	   | Some p' ->
	      p'#removeChild currentNode';
	      currentNode
	 else
	   let (nsn, nsl) = currentNode'#getNextSibling in
	   removeNamedItemInternal (nsn)
    in
    let (fcn,fcl) = rootNode#getFirstChild in
    removeNamedItemInternal fcn

  method item (index:int) =
    let rec itemInternal (i:int) (ind:int) (currentNode:node' option)= 
      match currentNode with
      | None -> None
      | Some currentNode' ->
	 let (nt,nl) = currentNode'#getnType in
	 if i = ind && nt = ATTRIBUTE_NODE then
	   currentNode
	 else
	   let (nsn, nsl) = currentNode'#getNextSibling in
	   itemInternal (i+1) (ind) (nsn)
    in
    let (fcn,fcl) = rootNode#getFirstChild in
    itemInternal 0 (index) fcn

  method getNamedItemNS (namespaceURI:domString) (localName:domString) = 
    let rec getNamedItemNSInternal (currentNode:node' option) = 
      match currentNode with
      | None -> None
      | Some currentNode' -> 
	 let (nName,nLabel) = currentNode'#getNamespaceURI in
	 let (lName,lLabel) = currentNode'#getLocalName in
	 let (nt,nl) = currentNode'#getnType in
	 if namespaceURI = nName && lName = localName && nt = ATTRIBUTE_NODE then 
	   currentNode
	 else
	   let (nsn, nsl) = currentNode'#getNextSibling in
	   getNamedItemNSInternal (nsn)
    in
    let (fcn,fcl) = rootNode#getFirstChild in
    getNamedItemNSInternal fcn

  method setNamedItemNS (arg:node') =
    let rec setNamedItemNSInternal (currentNode:node' option) = 
      match currentNode with
      | None -> None
      | Some currentNode' -> 
	 let (cnName,cnLabel) = currentNode'#getNamespaceURI in
	 let (clName,clLabel) = currentNode'#getLocalName in
	 let (nName,nLabel) = arg#getNamespaceURI in
	 let (lName,lLabel) = arg#getLocalName in
	 let (nt,nl) = currentNode'#getnType in
	 if  cnName = nName && clName = lName && nt = ATTRIBUTE_NODE then 
	   let (p,pl) = currentNode'#getParentNode in
	   match p with
	   | None -> None
	   | Some p' ->
	      p'#replaceChild currentNode' arg;
	      currentNode
	 else
	   let (nsn, nsl) = currentNode'#getNextSibling in
	   setNamedItemNSInternal (nsn)
    in
    let (fcn,fcl) = rootNode#getFirstChild in
    setNamedItemNSInternal fcn

  method removeNamedItemNS (namespaceURI:domString) (localName:domString) =
    let rec removeNamedItemNSInternal (currentNode:node' option) = 
      match currentNode with
      | None -> None
      | Some currentNode' -> 
	 let (nName,nLabel) = currentNode'#getNamespaceURI in
	 let (lName,lLabel) = currentNode'#getLocalName in
	 let (nt,nl) = currentNode'#getnType in
	 if  nName = namespaceURI && lName = localName && nt = ATTRIBUTE_NODE then 
	   let (p,pl) = currentNode'#getParentNode in
	   match p with
	   | None -> None
	   | Some p' ->
	      p'#removeChild currentNode';
	      currentNode
	 else
	   let (nsn, nsl) = currentNode'#getNextSibling in
	   removeNamedItemNSInternal (nsn)
    in
    let (fcn,fcl) = rootNode#getFirstChild in
    removeNamedItemNSInternal fcn
end
(*************************************************************************)
and entitynamedNodeMap (root:node') = object (self)
inherit newnamedNodeMap

  val rootNode = root

  method getNamedItem (name:domString) = 
    let rec getNamedItemInternal (currentNode:node' option) = 
      match currentNode with
      | None -> None
      | Some currentNode' -> 
	 let (nName,nLabel) = currentNode'#getNodeName in
	 let (nt,nl) = currentNode'#getnType in
	 if name =  nName && nt = ENTITY_NODE then 
	   currentNode
	 else
	   let (nsn, nsl) = currentNode'#getNextSibling in
	   getNamedItemInternal (nsn)
    in
    let (fcn,fcl) = rootNode#getFirstChild in
    getNamedItemInternal fcn

  method setNamedItem (ard:node') =
    let rec setNamedItemInternal (currentNode:node' option) = 
      match currentNode with
      | None -> None
      | Some currentNode' -> 
	 let (nName,nLabel) = currentNode'#getNodeName in
	 let (aName,aLabel) = ard#getNodeName in
	 let (nt,nl) = currentNode'#getnType in
	 if  nName = aName &&  nt = ENTITY_NODE then 
	   let (p,pl) = currentNode'#getParentNode in
	   match p with
	   | None -> None
	   | Some p' ->
	      p'#replaceChild currentNode' ard;
	      currentNode
	 else
	   let (nsn, nsl) = currentNode'#getNextSibling in
	   setNamedItemInternal (nsn)
    in
    let (fcn,fcl) = rootNode#getFirstChild in
    setNamedItemInternal fcn

  method removeNamedItem (name:domString) =
    let rec removeNamedItemInternal (currentNode:node' option) = 
      match currentNode with
      | None -> None
      | Some currentNode' -> 
	 let (nName,nLabel) = currentNode'#getNodeName in
	 let (nt,nl) = currentNode'#getnType in
	 if  nName = name &&  nt = ENTITY_NODE then 
	   let (p,pl) = currentNode'#getParentNode in
	   match p with
	   | None -> None
	   | Some p' ->
	      p'#removeChild currentNode';
	      currentNode
	 else
	   let (nsn, nsl) = currentNode'#getNextSibling in
	   removeNamedItemInternal (nsn)
    in
    let (fcn,fcl) = rootNode#getFirstChild in
    removeNamedItemInternal fcn

  method item (index:int) =
    let rec itemInternal (i:int) (ind:int) (currentNode:node' option)= 
      match currentNode with
      | None -> None
      | Some currentNode' -> 
	 let (nt,nl) = currentNode'#getnType in
	 if i = ind && nt = ENTITY_NODE then
	   currentNode
	 else
	   let (nsn, nsl) = currentNode'#getNextSibling in
	   itemInternal (i+1) (ind) (nsn)
    in
    let (fcn,fcl) = rootNode#getFirstChild in
    itemInternal 0 (index) fcn

  method getNamedItemNS (namespaceURI:domString) (localName:domString) = 
    let rec getNamedItemNSInternal (currentNode:node' option) = 
      match currentNode with
      | None -> None
      | Some currentNode' -> 
	 let (nName,nLabel) = currentNode'#getNamespaceURI in
	 let (lName,lLabel) = currentNode'#getLocalName in
	 let (nt,nl) = currentNode'#getnType in
	 if namespaceURI = nName && lName = localName && nt = ENTITY_NODE then 
	   currentNode
	 else
	   let (nsn, nsl) = currentNode'#getNextSibling in
	   getNamedItemNSInternal (nsn)
    in
    let (fcn,fcl) = rootNode#getFirstChild in
    getNamedItemNSInternal fcn

  method setNamedItemNS (arg:node') =
    let rec setNamedItemNSInternal (currentNode:node' option) = 
      match currentNode with
      | None -> None
      | Some currentNode' -> 
	 let (cnName,cnLabel) = currentNode'#getNamespaceURI in
	 let (clName,clLabel) = currentNode'#getLocalName in
	 let (nName,nLabel) = arg#getNamespaceURI in
	 let (lName,lLabel) = arg#getLocalName in
	 let (nt,nl) = currentNode'#getnType in
	 if  cnName = nName && clName = lName && nt = ENTITY_NODE then 
	   let (p,pl) = currentNode'#getParentNode in
	   match p with
	   | None -> None
	   | Some p' ->
	      p'#replaceChild currentNode' arg;
	      currentNode
	 else
	   let (nsn, nsl) = currentNode'#getNextSibling in
	   setNamedItemNSInternal (nsn)
    in
    let (fcn,fcl) = rootNode#getFirstChild in
    setNamedItemNSInternal fcn

  method removeNamedItemNS (namespaceURI:domString) (localName:domString) =
    let rec removeNamedItemNSInternal (currentNode:node' option) = 
      match currentNode with
      | None -> None
      | Some currentNode' -> 
	 let (nName,nLabel) = currentNode'#getNamespaceURI in
	 let (lName,lLabel) = currentNode'#getLocalName in
	 let (nt,nl) = currentNode'#getnType in
	 if  nName = namespaceURI && lName = localName && nt = ENTITY_NODE then 
	   let (p,pl) = currentNode'#getParentNode in
	   match p with
	   | None -> None
	   | Some p' ->
	      p'#removeChild currentNode';
	      currentNode
	 else
	   let (nsn, nsl) = currentNode'#getNextSibling in
	   removeNamedItemNSInternal (nsn)
    in
    let (fcn,fcl) = rootNode#getFirstChild in
    removeNamedItemNSInternal fcn
end
(*************************************************************************)
and notationnamedNodeMap (root:node') = object (self)
inherit newnamedNodeMap
  val rootNode = root

  method getNamedItem (name:domString) = 
    let rec getNamedItemInternal (currentNode:node' option) = 
      match currentNode with
      | None -> None
      | Some currentNode' -> 
	 let (nName,nLabel) = currentNode'#getNodeName in
	 let (nt,nl) = currentNode'#getnType in
	 if name =  nName && nt = NOTATION_NODE then 
	   currentNode
	 else
	   let (nsn, nsl) = currentNode'#getNextSibling in
	   getNamedItemInternal (nsn)
    in
    let (fcn,fcl) = rootNode#getFirstChild in
    getNamedItemInternal fcn

  method setNamedItem (ard:node') =
    let rec setNamedItemInternal (currentNode:node' option) = 
      match currentNode with
      | None -> None
      | Some currentNode' -> 
	 let (nName,nLabel) = currentNode'#getNodeName in
	 let (aName,aLabel) = ard#getNodeName in
	 let (nt,nl) = currentNode'#getnType in
	 if  nName = aName &&  nt = NOTATION_NODE then 
	   let (p,pl) = currentNode'#getParentNode in
	   match p with
	   | None -> None
	   | Some p' ->
	      p'#replaceChild currentNode' ard;
	      currentNode
	 else
	   let (nsn, nsl) = currentNode'#getNextSibling in
	   setNamedItemInternal (nsn)
    in
    let (fcn,fcl) = rootNode#getFirstChild in
    setNamedItemInternal fcn

  method removeNamedItem (name:domString) =
    let rec removeNamedItemInternal (currentNode:node' option) = 
      match currentNode with
      | None -> None
      | Some currentNode' -> 
	 let (nName,nLabel) = currentNode'#getNodeName in
	 let (nt,nl) = currentNode'#getnType in
	 if  nName = name &&  nt = NOTATION_NODE then 
	   let (p,pl) = currentNode'#getParentNode in
	   match p with
	   | None -> None
	   | Some p' ->
	      p'#removeChild currentNode';
	      currentNode
	 else
	   let (nsn, nsl) = currentNode'#getNextSibling in
	   removeNamedItemInternal (nsn)
    in
    let (fcn,fcl) = rootNode#getFirstChild in
    removeNamedItemInternal fcn

  method item (index:int) =
    let rec itemInternal (i:int) (ind:int) (currentNode:node' option)= 
      match currentNode with
      | None -> None
      | Some currentNode' -> 
	 let (nt,nl) = currentNode'#getnType in
	 if i = ind && nt = NOTATION_NODE then
	   currentNode
	 else
	   let (nsn, nsl) = currentNode'#getNextSibling in
	   itemInternal (i+1) (ind) (nsn)
    in
    let (fcn,fcl) = rootNode#getFirstChild in
    itemInternal 0 (index) fcn

  method getNamedItemNS (namespaceURI:domString) (localName:domString) = 
    let rec getNamedItemNSInternal (currentNode:node' option) = 
      match currentNode with
      | None -> None
      | Some currentNode' -> 
	 let (nName,nLabel) = currentNode'#getNamespaceURI in
	 let (lName,lLabel) = currentNode'#getLocalName in
	 let (nt,nl) = currentNode'#getnType in
	 if namespaceURI = nName && lName = localName && nt = NOTATION_NODE then 
	   currentNode
	 else
	   let (nsn, nsl) = currentNode'#getNextSibling in
	   getNamedItemNSInternal (nsn)
    in
    let (fcn,fcl) = rootNode#getFirstChild in
    getNamedItemNSInternal fcn

  method setNamedItemNS (arg:node') =
    let rec setNamedItemNSInternal (currentNode:node' option) = 
      match currentNode with
      | None -> None
      | Some currentNode' -> 
	 let (cnName,cnLabel) = currentNode'#getNamespaceURI in
	 let (clName,clLabel) = currentNode'#getLocalName in
	 let (nName,nLabel) = arg#getNamespaceURI in
	 let (lName,lLabel) = arg#getLocalName in
	 let (nt,nl) = currentNode'#getnType in
	 if  cnName = nName && clName = lName && nt = NOTATION_NODE then 
	   let (p,pl) = currentNode'#getParentNode in
	   match p with
	   | None -> None
	   | Some p' ->
	      p'#replaceChild currentNode' arg;
	      currentNode
	 else
	   let (nsn, nsl) = currentNode'#getNextSibling in
	   setNamedItemNSInternal (nsn)
    in
    let (fcn,fcl) = rootNode#getFirstChild in
    setNamedItemNSInternal fcn

  method removeNamedItemNS (namespaceURI:domString) (localName:domString) =
    let rec removeNamedItemNSInternal (currentNode:node' option) = 
      match currentNode with
      | None -> None
      | Some currentNode' -> 
	 let (nName,nLabel) = currentNode'#getNamespaceURI in
	 let (lName,lLabel) = currentNode'#getLocalName in
	 let (nt,nl) = currentNode'#getnType in
	 if  nName = namespaceURI && lName = localName && nt = NOTATION_NODE then 
	   let (p,pl) = currentNode'#getParentNode in
	   match p with
	   | None -> None
	   | Some p' ->
	      p'#removeChild currentNode'; 
	      currentNode
	 else
	   let (nsn, nsl) = currentNode'#getNextSibling in
	   removeNamedItemNSInternal (nsn)
    in
    let (fcn,fcl) = rootNode#getFirstChild in
    removeNamedItemNSInternal fcn
end
(*************************************************************************)
and domStringList = object (self)
    val mutable domStringList: domString list = []
    val mutable length = 0

    method item (index:int):domString = 
      List.nth domStringList index

    method private containsInternal (str:domString) (strList:domString list):bool = 
      match strList with  
      | nil -> false
      | h::strList' -> 
	 if h=str then true
	 else self#containsInternal str strList'

    method contains (str:domString):bool =
      self#containsInternal str domStringList
end
(*************************************************************************)
and entity = object(self)
inherit node
   val mutable publicID = ""
   val mutable publicIDLabel: labelClass = pcLabel
   method getpublicId = (publicID, publicIDLabel)
   method setpublicId (p,l) = 
     if nsuCheck publicIDLabel pcLabel =false then
      let _ = hang() in ()
     else
       ( publicIDLabel <- (pcLabel#join l); publicID <- p )

   val mutable systemID = ""
   val mutable systemIDLabel: labelClass = pcLabel
   method getsystemId = (systemID, systemIDLabel)
   method setsystemId (s,l) = 
     if nsuCheck systemIDLabel pcLabel =false then
      let _ = hang() in ()
     else
       ( systemIDLabel <- (pcLabel#join l); systemID <- s )

   val mutable notationName = ""
   val mutable notationNameLabel: labelClass = pcLabel
   method getnotationName = (notationName, notationNameLabel)
   method setnotationName (n,l) = 
     if nsuCheck notationNameLabel pcLabel =false then
       let _ = hang() in ()
     else
       ( notationNameLabel <- (pcLabel#join l); notationName <- n )

   val mutable inputEncoding = ""
   val mutable inputEncodingLabel: labelClass = pcLabel
   method getinputEncoding = (inputEncoding, inputEncodingLabel)
   method setinputEncoding (i,l) = 
     if nsuCheck inputEncodingLabel pcLabel =false then
      let _ = hang() in ()
     else
       ( inputEncodingLabel  <- (pcLabel#join l); inputEncoding <- i )

   val mutable xmlEncoding = ""
   val mutable xmlEncodingLabel: labelClass = pcLabel
   method getxmlEncoding = (xmlEncoding, xmlEncodingLabel)
   method setxmlEncoding (x,l) = 
     if nsuCheck xmlEncodingLabel pcLabel =false then
      let _ = hang() in ()
     else
       ( xmlEncodingLabel <- (pcLabel#join l); xmlEncoding <- x )

   val mutable xmlVersion = ""
   val mutable xmlVersionLabel: labelClass = pcLabel
   method getxmlVersion = (xmlVersion, xmlVersionLabel)
   method setxmlVersion (v,l) = 
     if nsuCheck (self#getNodeLabel) pcLabel =false then
      let _ = hang() in ()
     else
       ( xmlVersionLabel <- (pcLabel#join l); xmlVersion <- v )
end
(*************************************************************************)
and entityReference:entityReference' = object (self)
inherit node
end
(*************************************************************************)
and processingInstruction:processingInstruction' = object(self)
inherit node
   val mutable target = ""
   val mutable targetLabel: labelClass = pcLabel
   method getTarget = (target, targetLabel)
   method setTarget (v,l) = 
     if nsuCheck targetLabel pcLabel =false then
      let _ = hang() in ()
     else
       ( targetLabel <- (pcLabel#join l); target <- v )

   val mutable data = ""
   val mutable dataLabel: labelClass = pcLabel
   method getData = (data, dataLabel)
   method setData (v,l) = 
     if nsuCheck (self#getNodeLabel) pcLabel =false then
      let _ = hang() in ()
     else
       ( dataLabel <- (pcLabel#join l); data <- v )
end
(*************************************************************************)
and notation = object(self)
inherit node
   val mutable publicID = ""
   val mutable publicIDLabel: labelClass = pcLabel
   method getpublicId = (publicID, publicIDLabel)
   method setpublicId (p,l) = 
     if nsuCheck (self#getNodeLabel) pcLabel =false then
      let _ = hang() in ()
     else
       ( publicIDLabel <- (pcLabel#join l); publicID <- p )

   val mutable systemID = ""
   val mutable systemIDLabel: labelClass = pcLabel
   method getsystemId = (systemID, systemIDLabel)
   method setsystemId (s,l) = 
     if nsuCheck systemIDLabel pcLabel =false then
       let _ = hang() in ()
     else
       ( systemIDLabel <- (pcLabel#join l); systemID <- s )
end;;
(*************************************************************************)
class domError = object(self)
  val mutable name = ""
  val mutable nameLabel: labelClass = pcLabel
 
  method getname = (name, nameLabel)
  method setname (n,l) = 
    if nsuCheck nameLabel pcLabel =false then
      let _ = hang() in 
      () 
    else
      ( nameLabel <- (pcLabel#join l); name <- n )
end;;
(*************************************************************************)
class nameList = object(self)
  val mutable nList = [("","")]
			
  method private insertTuple (tup) =
    List.append nList [tup]
		
  method private getTupleList = nList

  method getname (n:int)= 
    let (name, namspaceURI) = List.nth nList n in
    name

  method getNamespaceURI (n:int)= 
    let (name, namespaceURI) = List.nth nList n in
    namespaceURI

  method  private containsName (str:domString) (l):bool = 
    match l with
    | nil -> false
    | (h1,h2)::l' -> 
       if h1=str then 
	 true
       else
	 self#containsName str l'

  method contains (str:domString): bool = 
    self#containsName str nList
		      
  method  private containsTuple (uri:domString) (str:domString) (l):bool = 
    match l with
    | nil -> false
    | (h1,h2)::l' -> 
       if h1=uri && h2=str then 
	 true
       else
	 self#containsTuple uri str l'

  method containsNS (namespaceURI:domString) (name:domString): bool = 
    self#containsTuple namespaceURI name nList 
end;;
(*************************************************************************)

(*************************Meta functions for semantic rules***************************************)
let getPathLabel (o:node') = 
  let rec getPathLabelInternal (p:node' option) (l:labelClass) =
    match p with
    | None -> l
    | Some p' ->
       let (pp,ppl) = p'#getParentNode in
       getPathLabelInternal pp (l#join ppl)
  in
  let (op,opl) = o#getParentNode in
  getPathLabelInternal (op) (opl)

let getAncestors (o:node') = 
  let rec getAncestorsInternal (p:node' option) (aList: node' list)=
    match p with
    | None -> aList
    | Some p' ->
       let (pp,ppl) = p'#getParentNode in
       getAncestorsInternal pp aList@[p']
  in
  let (op,opl) = o#getParentNode in
  getAncestorsInternal (op) ([o])

let getDefaults (l:node' list) (n:node') (bubbles:bool) =
  if bubbles = true then
    [n]@l
  else
    [n]

let getEventHandlers (e:event) (n:node') (p:eventPhase) = 
  let (evnType,el) = e#getEventType in
  let toEventTarget (no:node') = (no :> eventTarget) in
  let nT = toEventTarget n in
  let hList = nT#getEventHandlers evnType in
  let rec getHandlersInternal (cList:(eventListener * bool) list) (l:eventListener list) = 
    match cList with
    | [] -> Some l
    | (handler,cflag)::cList' -> 
       match p with
       | START_PHASE -> None
       | CAPTURE_PHASE -> 
	  if cflag = true then
	    getHandlersInternal (cList') (l@[handler])
	  else
	    getHandlersInternal (cList') (l)
       | TARGET_PHASE -> Some l
       | BUBBLE_PHASE -> 
	  if cflag = true then
	    getHandlersInternal (cList') (l)
	  else
	    getHandlersInternal (cList') (l@[handler])
       | DEFAULT_PHASE -> None
  in 
  getHandlersInternal hList []

(*************************************************************************)
